<?php
/*
 * $Horde: imp/compose.php,v 2.249 2001/01/10 19:24:21 chuck Exp $
 *
 * Copyright 1999, 2000 Charles J. Hagenbuch <chuck@horde.org>
 * Copyright 1999, 2000 Jon Parise <jon@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

/*
 * Formats the From: address correctly, based on all of the relevant factors.
 */
function get_from()
{
    global $conf, $imp, $prefs, $from;

    $name = $prefs->getValue('fullname');
    if (empty($name)) {
        $name = '';
    }

    if (!isset($from) || !$from || !$prefs->isChangeable('from_addr')) {
        $from = '';
        if ($name) {
            $from  = $name;
            $from .= ' <';
        }
        $from .= get_barefrom();
        if ($name) {
            $from .= '>'; 
        }
    }
  
    return $from;
}

/**
 * Return an url for mailbox.php depending on what kind of start/page
 * information we have.
 */
function mailboxReturnURL()
{
    global $HTTP_POST_VARS, $HTTP_GET_VARS;
    
    $url = 'mailbox.php?';
    
    if (isset($HTTP_POST_VARS['start'])) {
        $url .= 'start=' . urlencode($HTTP_POST_VARS['start']) . '&';
    } else if (isset($HTTP_GET_VARS['start'])) {
        $url .= 'start=' . urlencode($HTTP_GET_VARS['start']) . '&';
    }
    
    if (isset($HTTP_POST_VARS['page'])) {
        $url .= 'page=' . urlencode($HTTP_POST_VARS['page']) . '&';
    } else if (isset($HTTP_GET_VARS['page'])) {
        $url .= 'page=' . urlencode($HTTP_GET_VARS['page']) . '&';
    }
    
    return substr($url, 0, -1);
}

/*
 * TODO: write function description
 */
function find_text_part(&$attachments)
{
    foreach ($attachments as $mime) {
        if ($mime->TYPE == TYPETEXT) {
            return IMP_Util::getDecodedPart($mime);
        }
    }

    return '';
}

/*
 * Return the value of upload_tmp_dir, or default to '/tmp'
 */
function tmpdir()
{
    $tmp = ini_get('upload_tmp_dir');
    if (empty($tmp)) {
        $tmp = getenv('TMPDIR');
    }
    return (empty($tmp) ? '/tmp' : $tmp);
}

/*
 * Make sure a path is inside of the upload directory.
 */
function safe_file($file) {
    return tmpdir() . '/' . basename($file);
}

/*
 * Checks for non-standard address formats, such as separating with
 * spaces or semicolons.
 */
function format_addresses($address_string)
{
    /*
     * If there are angle brackets (<>), or a colon (group name
     * delimiter), assume the user knew what they were doing.
     */
    if ((strpos($address_string, '>') === false) &&
        (strpos($address_string, ':') === false)){
        $address_string = trim(strtr($address_string, ';,', '  '));
        $address_string = preg_replace('|\s+|', ', ', $address_string);
    }
    return $address_string;
}

/*
 * Encodes attachments for a message being sent or saved.
 */
function addMimeParts(&$message)
{
    global $HTTP_POST_VARS, $HTTP_GET_VARS, $imp;
    
    if (isset($HTTP_POST_VARS['attachments_name']) && count($HTTP_POST_VARS['attachments_name']) > 0) {
        $index = Horde::getFormData('index');
        if (!empty($index)) {
            include_once '../lib/MIME/Structure.php';
            $structure = imap_fetchstructure($imp['stream'], $index, FT_UID);
            $attachments = MIME_Structure::parse($structure, $index, $MimeID);
        }
        
        for ($i = 0; $i < count($HTTP_POST_VARS['attachments_name']); $i++) {
            /* Read it from the filesystem if it's a normal attachment. */
            if (!preg_match('|.*-(.*).mime$|', $HTTP_POST_VARS['attachments_file'][$i], $regs)) {
                $fd = fopen(safe_file($HTTP_POST_VARS['attachments_file'][$i]), 'r');
                $contents = fread($fd, $HTTP_POST_VARS['attachments_size'][$i]);
                fclose($fd); 
                unlink(safe_file($HTTP_POST_VARS['attachments_file'][$i]));
            } else {
                /* Fetch it from the mail message content if it's a
                 * forwarded attachment. */
                $ref = $regs[1];
                $fmime = $attachments[$ref];
                $contents = IMP_Util::getPart($fmime);
            }
            
            include_once '../lib/MIME/Message.php';
            $file_part = new MIME_Part();
            
            /* Some browsers do not send the mime type. */
            if (strcmp($HTTP_POST_VARS['attachments_type'][$i], '')) { 
                $file_part->setType($HTTP_POST_VARS['attachments_type'][$i]);
            } else {
                $file_part->setType('application/octet-stream');
            }
            
            $file_part->setName($HTTP_POST_VARS['attachments_name'][$i]);
            
            /* If the type isn't text, we need to base64 it and mark
               that as the transfer encoding. */
            if ($file_part->type != 'text') {
                $contents = chunk_split(base64_encode($contents), 73, "\n") . "\n";
                $file_part->setTransferEncoding('base64');
            }
            $file_part->setContents($contents);
            $message->addPart($file_part);
        }
    }
}

/*
 * Adds the attachments to the message (in the case of a forward with
 * attachments).
 */
function attach_files($structure, $ref)
{
    global $HTTP_POST_VARS, $HTTP_GET_VARS;
	
    if (isset($HTTP_POST_VARS['attachments_name']) && !is_array($HTTP_POST_VARS['attachments_name'])) {
        $HTTP_POST_VARS['attachments_name'] = array($HTTP_POST_VARS['attachments_name']);
    }
    if (isset($HTTP_POST_VARS['attachments_size']) && !is_array($HTTP_POST_VARS['attachments_size'])) {
        $HTTP_POST_VARS['attachments_size'] = array($HTTP_POST_VARS['attachments_size']);
    }
    if (isset($HTTP_POST_VARS['attachments_file']) && !is_array($HTTP_POST_VARS['attachments_file'])) {
        $HTTP_POST_VARS['attachments_file'] = array($HTTP_POST_VARS['attachments_file']);
    }
    if (isset($HTTP_POST_VARS['attachments_type']) && !is_array($HTTP_POST_VARS['attachments_type'])) {
        $HTTP_POST_VARS['attachments_type'] = array($HTTP_POST_VARS['attachments_type']);
    }
    
    include_once '../lib/MIME/Structure.php';
    $attachments = MIME_Structure::parse($structure, Horde::getFormData('index'), $MimeID);
    while (list($ref, $mime) = each($attachments)) {
        if (!$mime->header) {
            $HTTP_POST_VARS['attachments_name'][] = $mime->name;
            $HTTP_POST_VARS['attachments_size'][] = $mime->size;
            $HTTP_POST_VARS['attachments_file'][] = $HTTP_GET_VARS['index'] . '-' . $ref . '.mime';
            $HTTP_POST_VARS['attachments_type'][] = $mime->TYPE . '/' . $mime->subtype;
        }
    }
}

require_once './lib/base.php';
require_once '../lib/Registry.php';
require_once '../lib/MIME.php';
require_once './lib/folder.php';
require './config/mime.php';
require './lib/mimetypes.php';

$registry = new Registry();

if ($conf['log']['enabled']) {
    include_once 'Log.php';
    $log = &Log::singleton($conf['log']['type'], $conf['log']['name'],
                           $conf['log']['ident'], $conf['log']['conf']);
}

session_name($horde['session_name']);
if ($browser->isBrowser('netscape')) {
    session_cache_limiter('public');
}
@session_start();

if (!isset($HTTP_SESSION_VARS['imp']) || !is_array($HTTP_SESSION_VARS['imp'])) {
    if (!empty($HTTP_GET_VARS['popup'])) {
        echo '<script language="JavaScript" type="text/javascript">window.close()</script>';
    } else {
        header('Location: ' . Horde::url('login.php?reason=logout', true));
    }
    exit;
}
IMP::setupSession();
IMP::authenticate();

/* Array to translate string actions into their respective integer ID's. */
$actions[_("Send Message")] = SEND_MESSAGE;
$actions[htmlentities(_("Send Message"))] = SEND_MESSAGE;

$actions[_("Attach")] = ADD_ATTACHMENT;
$actions[htmlentities(_("Attach"))] = ADD_ATTACHMENT;

$actions[_("Delete Attachments")] = DELETE_ATTACHMENT;
$actions[htmlentities(_("Delete Attachments"))] = DELETE_ATTACHMENT;

$actions[_("Bounce Message")] = BOUNCE_MESSAGE;
$actions[htmlentities(_("Bounce Message"))] = BOUNCE_MESSAGE;

$actions[_("Save Draft")] = SAVE_DRAFT;
$actions[htmlentities(_("Save Draft"))] = SAVE_DRAFT;

/* Filter form variables, if necessary. */
if (get_magic_quotes_gpc()) {
    if (isset($HTTP_POST_VARS['message']))
        $HTTP_POST_VARS['message'] = stripslashes($HTTP_POST_VARS['message']);
    elseif (isset($HTTP_GET_VARS['message']))
        $HTTP_GET_VARS['message'] = stripslashes($HTTP_GET_VARS['message']);
    
    if (isset($HTTP_POST_VARS['msg']))
        $HTTP_POST_VARS['msg'] = stripslashes($HTTP_POST_VARS['msg']);
    elseif (isset($HTTP_GET_VARS['msg']))
        $HTTP_GET_VARS['msg'] = stripslashes($HTTP_GET_VARS['msg']);
    
    if (isset($HTTP_POST_VARS['subject']))
        $HTTP_POST_VARS['subject'] = stripslashes($HTTP_POST_VARS['subject']);
    elseif (isset($HTTP_GET_VARS['subject']))
        $HTTP_GET_VARS['subject'] = stripslashes($HTTP_GET_VARS['subject']);
    
    if (isset($HTTP_POST_VARS['from']))
        $HTTP_POST_VARS['from'] = stripslashes($HTTP_POST_VARS['from']);
    elseif (isset($HTTP_GET_VARS['from']))
        $HTTP_GET_VARS['from'] = stripslashes($HTTP_GET_VARS['from']);
    
    if (isset($HTTP_POST_VARS['to']))
        $HTTP_POST_VARS['to'] = stripslashes($HTTP_POST_VARS['to']);
    elseif (isset($HTTP_GET_VARS['to']))
        $HTTP_GET_VARS['to'] = stripslashes($HTTP_GET_VARS['to']);
    
    if (isset($HTTP_POST_VARS['cc']))
        $HTTP_POST_VARS['cc'] = stripslashes($HTTP_POST_VARS['cc']);
    elseif (isset($HTTP_GET_VARS['cc']))
        $HTTP_GET_VARS['cc'] = stripslashes($HTTP_GET_VARS['cc']);
    
    if (isset($HTTP_POST_VARS['bcc']))
        $HTTP_POST_VARS['bcc'] = stripslashes($HTTP_POST_VARS['bcc']);
    elseif (isset($HTTP_GET_VARS['bcc']))
        $HTTP_GET_VARS['bcc'] = stripslashes($HTTP_GET_VARS['bcc']);
}

$get_sig = true;

/* Get the 'save_sent_mail' setting from the preferences if it's not set. */
if (!isset($HTTP_POST_VARS['save_sent_mail'])) {
    $HTTP_POST_VARS['save_sent_mail'] = $prefs->getValue('save_sent_mail');
}

/* Get the 'sent_mail_folder' value from the preferences if it's not set. */
if (!isset($sent_mail_folder)) {
    $sent_mail_folder = $prefs->getValue('sent_mail_folder');
} else {
    $sent_mail_folder = urldecode($sent_mail_folder);
}

// TODO: Find a better way to handle the following two conversions.

/* Strip off the namespace prefix; we'll add it later. */
if (strlen($conf['server']['namespace']) > 0) {
    $sent_mail_folder = str_replace($conf['server']['namespace'], '', $sent_mail_folder);
}

/* Strip off the folders prefix; we'll add it later. */
if (strlen($imp['folders']) > 0) {
    $sent_mail_folder = str_replace($imp['folders'], '', $sent_mail_folder);
}

$actionID = IMP_Util::getAction();

/* Translate non-int values into the correct int. */
if (strcspn($actionID, '0123456789')) {
    $actionID = $actions[$actionID];
}

/* Run through the action handlers. */
switch ($actionID) {
case NO_ACTION:
    $ACTION_TEXT = _("Message Composition");
    break;
    
case COMPOSE:
    if (isset($HTTP_GET_VARS['to'])) {
        $to = MIME::decode($HTTP_GET_VARS['to']);
    }
    if (isset($HTTP_GET_VARS['subject'])) {
        $subject = MIME::decode($HTTP_GET_VARS['subject']);
    }
    $ACTION_TEXT = _("Compose a message");
    break;
    
case MAILTO:
    if (!empty($HTTP_GET_VARS['index'])) {
        $h = @imap_header($imp['stream'], imap_msgno($imp['stream'], $HTTP_GET_VARS['index']));
        if (isset($mailto) && isset($h->$mailto)) {
            $to = MIME::decode($h->$mailto);
        } else if (!empty($h->reply_to)) {
            $to = IMP::addrArray2String($h->reply_to);
        } else if (isset($h->from)) {
            $to = IMP::addrArray2String($h->from);
        } else {
            $to = '';
        }
        $ACTION_TEXT = _("Compose a message");
    }
    break;
    
case DRAFT:
    if (!empty($HTTP_GET_VARS['index'])) {
        $h = imap_header($imp['stream'], imap_msgno($imp['stream'], $HTTP_GET_VARS['index']));
        
        if (is_object($h)) {
            if (isset($h->to)) {
                $to = IMP::addrArray2String($h->to);
            }
            if (isset($h->cc)) {
                $cc = IMP::addrArray2String($h->cc);
            }
            if (isset($h->bcc)) {
                $bcc = IMP::addrArray2String($h->bcc);
            }
            if (isset($h->reply_to)) {
                $reply_to = IMP::addrArray2String($h->reply_to);
            }
            if (isset($h->subject)) {
                $subject = MIME::decode($h->subject);
            }
        }
        
        $msg = imap_fetchbody($imp['stream'], $HTTP_GET_VARS['index'], 1, FT_UID);
        $ACTION_TEXT = _("Compose a message");
        $structure = @imap_fetchstructure($imp['stream'], $HTTP_GET_VARS['index'], FT_UID);
        if (@count($structure->parts) > 1) {
            attach_files($structure, '');
        }
    }
    $get_sig = false;
    break;
    
case REPLY:
    if (!empty($HTTP_GET_VARS['index'])) {
        $h = imap_header($imp['stream'], imap_msgno($imp['stream'], $HTTP_GET_VARS['index']));
        
        /* Set the message_id and references headers. */
        if (isset($h->message_id)) {
            $in_reply_to = chop($h->message_id);
            if (isset($h->references)) {
                $references = chop($h->references) . ' ' . $in_reply_to;
            } else {
                $references = $in_reply_to;
            }
        } else {
            $in_reply_to = '';
            $references = '';
        }
        
        if (isset($h->reply_to)) {
            $to = IMP::addrArray2String($h->reply_to);
        } elseif (isset($h->from)) {
            $to = IMP::addrArray2String($h->from);
        } else {
            $to = '';
        }
        
        if (isset($h->from)) {
            $qfrom = IMP::addrArray2String($h->from);
        } else {
            $qfrom = '&lt;&gt;';
        }
        
        include_once '../lib/MIME/Structure.php';
        $structure = imap_fetchstructure($imp['stream'], $HTTP_GET_VARS['index'], FT_UID);
        $attachments = MIME_Structure::parse($structure, $HTTP_GET_VARS['index'], $MimeID);
        
        $quote_str = "\n" . $conf['msg']['quote_prefix'];
        $msg = find_text_part($attachments);
        $msg = Text::wrap($msg, (80 - strlen($quote_str)), $quote_str);
        $msg  = _("Quoting") . ' ' . $qfrom . ":\n$quote_str$msg";
        $msg .= "\n";
        
        if (isset($h->subject) && (strtolower(substr(MIME::decode($h->subject), 0, 3)) != 're:')) {
            $subject = 'Re: ' . MIME::decode($h->subject);
        } else if (isset($h->subject)) {
            $subject = MIME::decode($h->subject);
        } else {
            $subject = 'Re: ';
        }
        
        $ACTION_TEXT = _("Reply:") . ' ' . $subject;
    }
    break;
    
case REPLY_ALL:
    if (!empty($HTTP_GET_VARS['index'])) {
        $h = imap_header($imp['stream'], imap_msgno($imp['stream'], $HTTP_GET_VARS['index']));
        
        /* Set the message_id and references headers. */
        if (isset($h->message_id)) {
            $in_reply_to = chop($h->message_id);
            if (isset($h->references)) {
                $references = chop($h->references) . ' ' . $in_reply_to;
            } else {
                $references = $in_reply_to;
            }
        } else {
            $in_reply_to = '';
            $references = '';
        }
        
        // filter out our own address from the addresses we reply to
        $me = get_barefrom();
        
        // build the to: header
        if (isset($h->reply_to)) {
            $to_arr = $h->reply_to;
            $to = $reply = IMP::addrArray2String($to_arr);
            if (isset($h->from) && $reply != IMP::addrArray2String($h->from)) {
                $to_arr = array_merge($to_arr, $h->from);
                $to = IMP::addrArray2String($to_arr);
            }
        } else if (isset($h->from)) {
            $to = $reply = IMP::addrArray2String($h->from);
        } else {
            $to = $reply = '';
        }
        
        // build the cc: header
        $cc_arr = array();
        if ($reply != IMP::addrArray2String($h->to)) {
            $cc_arr = $h->to;
            if (isset($h->cc)) {
                $cc_arr = array_merge($cc_arr, $h->cc);
            }
        } else if (isset($h->cc)) {
            $cc_arr = $h->cc;
        }
        $cc = IMP::addrArray2String($cc_arr, $me);
        
        // build the bcc: header
        if (isset($h->bcc)) {
            $bcc = IMP::addrArray2String($h->bcc, $me);
        }
        
        include_once '../lib/MIME/Structure.php';
        $structure = imap_fetchstructure($imp['stream'], $HTTP_GET_VARS['index'], FT_UID);
        $attachments = MIME_Structure::parse($structure, $HTTP_GET_VARS['index'], $MimeID);
        
        $quote_str = "\n" . $conf['msg']['quote_prefix'];
        $msg = find_text_part($attachments);
        $msg = Text::wrap($msg, 80 - strlen($quote_str), $quote_str);
        $msg  = _("Quoting") . ' ' . IMP::addrObject2String($h->from[0]) . ":\n$quote_str$msg";
        $msg .= "\n";
        
        if (isset($h->subject) && strtolower(substr(MIME::decode($h->subject), 0, 3)) != 're:') {
            $subject = 'Re: ' . MIME::decode($h->subject);
        } else if (isset($h->subject)) {
            $subject = MIME::decode($h->subject);
        } else {
            $subject = '';
        }
        
        $ACTION_TEXT = _("Reply to All:") . ' ' . $subject;
    }
    break;
    
case FORWARD:
    if (!empty($HTTP_GET_VARS['index'])) {
        $h = imap_header($imp['stream'], imap_msgno($imp['stream'], $HTTP_GET_VARS['index']));
        if (isset($h->from[0])) {
            $message_source = IMP::addrObject2String($h->from[0]);
        } else {
            $message_source = '';
        }
        $barefrom = get_barefrom();
        $from = get_from();
        
        $msg = "\n\n";
        $msg .= "\n----- "; 
        $msg .= sprintf(_("Forwarded message from %s"), $message_source);
        $msg .= " -----\n";
        
        if (isset($h->date) && $h->date) {
            $msg .= _("    Date: ") . MIME::decode($h->date);
            $msg .= "\n";
        }
        if (isset($h->from) && is_array($h->from)) {
            $msg .= _("    From: ") . IMP::addrArray2String($h->from);
            $msg .= "\n";
        }
        if (isset($h->reply_to) && is_array($h->reply_to)) {
            $msg .= _("Reply-To: ") . IMP::addrArray2String($h->reply_to);
            $msg .= "\n";
        }
        if (isset($h->subject) && $h->subject) {
            $msg .= _(" Subject: ") . MIME::decode($h->subject);
            $msg .= "\n";
        }
        if (isset($h->to) && is_array($h->to)) {
            $msg .= _("      To: ") . IMP::addrArray2String($h->to);
            $msg .= "\n";
        }
        
        include_once '../lib/MIME/Structure.php';
        $structure = imap_fetchstructure($imp['stream'], $HTTP_GET_VARS['index'], FT_UID);
        $attachments = MIME_Structure::parse($structure, $HTTP_GET_VARS['index'], $MimeID);
        $msg .= "\n";
        $msg .= Text::wrap(find_text_part($attachments), 80 - 8);
        $msg .= "\n----- " . _("End forwarded message") . " -----\n";
        if ($qprint_msg = imap_qprint($msg)) {
            $msg = $qprint_msg;
        }
        
        if (isset($h->subject)) {
            $subject = _("Fwd:") . ' ' . MIME::decode($h->subject);
            $ACTION_TEXT = _("Forward:") . ' ' . MIME::decode($h->subject);
        } else {
            $subject = _("Fwd:");
            $ACTION_TEXT = _("Forward");
        }
        if (@count($structure->parts) > 1) {
            attach_files($structure, '');
        }
    }
    break;

case BOUNCE_MESSAGE:
    if (!empty($HTTP_POST_VARS['index']) && !empty($HTTP_POST_VARS['to'])) {
        $recipients = $HTTP_POST_VARS['to'];
        
        $orig_headers = imap_fetchheader($imp['stream'], $HTTP_POST_VARS['index'], FT_UID);
        $orig_headers = str_replace("\r", '', $orig_headers);
        $header_arr = explode("\n", $orig_headers);
        
        for ($i = 0; $i < count($header_arr); $i++) {
            if (!empty($header_arr[$i]) && $header_arr[$i] != '' && $header_arr[$i] != "\n") {
                if (preg_match('|^([-\w]*): (.*)|', $header_arr[$i], $pieces)) {
                    $headers[$pieces[1]] = $pieces[2];
                    $last = $pieces[1];
                } else {
                    if (isset($last) && $headers[$last]) {
                        $headers[$last] .= "\n" . $header_arr[$i];
                    }
                }
            }
        }
        
        $headers['Resent-Date'] = date('r');
        $headers['Resent-From'] = get_barefrom();
        $headers['Resent-To'] = $HTTP_POST_VARS['to'];
        $headers['Resent-Message-ID'] = '<' . uniqid(time() . '.') . '@' . $HTTP_SERVER_VARS['SERVER_NAME'] . '>';
        addSiteHeaders($headers);
        
        $msg = imap_body($imp['stream'], $HTTP_POST_VARS['index'], FT_UID);
        
        include_once 'Mail.php';
        $mailer = Mail::factory($conf['mailer']['type'], $conf['mailer']['params']);
        $status = $mailer->send($recipients, $headers, $msg);
        if (!PEAR::isError($status)) {
            if ($conf['log']['enabled']) {
                $entry = sprintf("%s Bounce message sent to %s from %s",
                        $HTTP_SERVER_VARS['REMOTE_ADDR'], $recipients, $imp['user']);
                $log->log($entry, LOG_INFO);
            }
            if ($prefs->getValue('compose_popup') && $browser->hasFeature('javascript')) {
                echo '<script language="JavaScript" type="text/javascript">window.close();</script>';
            } else {
                header('Location: ' . Horde::url(mailboxReturnURL()));
            }
            exit;
        }
        unset($HTTP_POST_VARS['to']);
        unset($msg);
    }
    break;
    
case SEND_MESSAGE:
    if (!empty($HTTP_POST_VARS['is_reply']) && isset($HTTP_POST_VARS['index'])) {
        imap_setflag_full($imp['stream'], $HTTP_POST_VARS['index'], '\\ANSWERED', SE_UID);
    }
    
    if (isset($attachments_name) && !is_array($attachments_name)) {
        $attachments = array($attachments_name);
    }
    if (isset($attachments_size) && !is_array($attachments_size)) {
        $attachments = array($attachments_size);
    }
    if (isset($attachments_file) && !is_array($attachments_file)) {
        $attachments = array($attachments_file);
    }
    if (isset($attachments_type) && !is_array($attachments_type)) {
        $attachments = array($attachments_type);
    }

    if (isset($HTTP_POST_VARS['message']) && (!empty($HTTP_POST_VARS['to']) || !empty($HTTP_POST_VARS['cc']) || !empty($HTTP_POST_VARS['bcc']))) {
        include_once '../lib/MIME/Message.php';
        $mime = new MIME_Message($imp['maildomain']);
        
        $HTTP_POST_VARS['message'] = str_replace("\r\n", "\n", $HTTP_POST_VARS['message']);
        $text_part = new MIME_Part('text/plain', $HTTP_POST_VARS['message']);
        if ($conf['msg']['append_trailer']) {
            /* Append footer to text_part. */
            $trailer = "\n";
            $trailer .= implode(@file('./config/trailer.txt'), '');
            $trailer = Text::expandEnvironment($trailer);
            $text_part->appendContents($trailer);
        }
        
        $mime->addPart($text_part);
        addMimeParts($mime);
        $msg = $mime->toString();
        
        $recipients = '';
        $barefrom = get_barefrom();
        $from = get_from();
        
        // add a Received header for the hop from browser to server.
        $remote = (!empty($HTTP_SERVER_VARS['REMOTE_HOST'])) ? $HTTP_SERVER_VARS['REMOTE_HOST'] : $HTTP_SERVER_VARS['REMOTE_ADDR'];
        $headers['Received'] = 'from ' . $remote . ' (';
        if (!empty($HTTP_SERVER_VARS['REMOTE_IDENT'])) $headers['Received'] .= $HTTP_SERVER_VARS['REMOTE_IDENT'] . '@' . $remote;
        $headers['Received'] .= ' [' . $remote . '])';
        $headers['Received'] .= "\n\t" . 'as user ' . $imp['user'] . '@' . $imp['server'];
        $headers['Received'] .= ' by ' . $HTTP_SERVER_VARS['SERVER_NAME'] . ' with HTTP;';
        $headers['Received'] .= "\n\t" . date('r');
        
        $headers['Message-ID'] = '<' . uniqid(time() . '.') . '@' . $HTTP_SERVER_VARS['SERVER_NAME'] . '>';
        $headers['Date'] = date('r');
        
        if ($conf['compose']['allow_receipts'] &&
            !empty($HTTP_POST_VARS['request_return_receipts'])) {
            $headers['Return-Receipt-To'] = $barefrom;
        }
        
        $headers['From'] = $from;
        if (!empty($HTTP_POST_VARS['to'])) {
            $HTTP_POST_VARS['to'] = format_addresses($HTTP_POST_VARS['to']);
            $headers['To'] = $HTTP_POST_VARS['to'];
            $recipients = $HTTP_POST_VARS['to'];
        }
        if ($conf['compose']['allow_cc'] && !empty($HTTP_POST_VARS['cc'])) {
            $cc = format_addresses($HTTP_POST_VARS['cc']);
            $headers['Cc'] = $HTTP_POST_VARS['cc'];
            $recipients = (empty($recipients)) ? $HTTP_POST_VARS['cc'] : "$recipients, " . $HTTP_POST_VARS['cc'];
        }
        if ($conf['compose']['allow_bcc'] && !empty($HTTP_POST_VARS['bcc'])) {
            $HTTP_POST_VARS['bcc'] = format_addresses($HTTP_POST_VARS['bcc']);
            $headers['Bcc'] = $HTTP_POST_VARS['bcc'];
            $recipients = (empty($recipients)) ? $HTTP_POST_VARS['bcc'] : "$recipients, " . $HTTP_POST_VARS['bcc'];
        }
        if (isset($HTTP_POST_VARS['subject']) && $HTTP_POST_VARS['subject']) {
            $headers['Subject'] = $HTTP_POST_VARS['subject'];
        }
        if (isset($references)) {
            $headers['References'] = $references;
        }
        if (isset($in_reply_to)) {
            $headers['In-Reply-To'] = $in_reply_to;
        }
        $headers = $mime->header($headers);
        $headers['User-Agent'] = IMP_NAME . ' ' . IMP_VERSION;
        addSiteHeaders($headers);
        
        include_once 'Mail.php';
        $mailer = Mail::factory($conf['mailer']['type'], $conf['mailer']['params']);
        $status = $mailer->send($recipients, $headers, $msg);
        if (!PEAR::isError($status)) {
            if ($conf['log']['enabled']) {
                $entry = sprintf("%s Message sent to %s from %s",
                        $HTTP_SERVER_VARS['REMOTE_ADDR'], $recipients, $imp['user']);
                $log->log($entry, LOG_INFO);
            }
            if ($conf['user']['allow_folders'] && $HTTP_POST_VARS['save_sent_mail']) {
                /* Loop through the envelope and add headers. */
                reset($headers);
                $fcc = '';
                while (list($key, $val) = each($headers)) {
                    if ($key != 'recipients') {
                        $fcc .= $key . ': ' . $val . "\n";
                    }
                }
                $fcc .= "\n";
                $fcc .= $msg;
                
                IMP_folder::create($imp['stream'], preambleString() . $sent_mail_folder, $prefs->getValue('subscribe'));
                $fcc = str_replace("\n", "\r\n", $fcc);
                if (!(imap_append($imp['stream'], serverString() . imap_utf7_encode(preambleString() . $sent_mail_folder), $fcc, '\\Seen'))) {
                    /* TODO: status(_("Message sent successfully, but not saved to") . $sent_mail_folder); */
                }
            }
            
            if ($prefs->getValue('compose_popup') && $browser->hasFeature('javascript')) {
                echo '<script language="JavaScript" type="text/javascript">window.close();</script>';
            } else {
                header('Location: ' . Horde::url(mailboxReturnURL()));
            }
            exit;
        }
    }
    $get_sig = false;
    $ACTION_TEXT = _("There was an error sending your message");
    $msg = $HTTP_POST_VARS['message'];
    break;

case SAVE_DRAFT:
    $append_flags = '\\Draft';

case POSTPONE_MESSAGE:
    $drafts_folder = $prefs->getValue('drafts_folder');
    if (!empty($drafts_folder)) {
        if ($HTTP_POST_VARS['message']) {
            $msg = $HTTP_POST_VARS['message']; // save for later.
            $HTTP_POST_VARS['message'] = str_replace("\r\n", "\n", $HTTP_POST_VARS['message']);
            
            include_once '../lib/MIME/Message.php';
            $mime = new MIME_Message($imp['maildomain']);
            
            $text_part = new MIME_Part('text/plain', $HTTP_POST_VARS['message']);
            $mime->addPart($text_part);
            addMimeParts($mime);
            $body = $mime->toString();
        } else {
            $body = '';
        }
        
        $barefrom = get_barefrom();
        $from = get_from();
        
        $hdrs = '';
        if (!empty($from)) {
            $hdrs .= "From: $from\n";
        }
        if (!empty($HTTP_POST_VARS['to'])) {
            $hdrs .= "To: " . $HTTP_POST_VARS['to'] . "\n";
        }
        if (!empty($HTTP_POST_VARS['cc'])) {
            $hdrs .= "Cc: " . $HTTP_POST_VARS['cc'] . "\n";
        }
        if (!empty($HTTP_POST_VARS['bcc'])) {
            $hdrs .= "Bcc: " . $HTTP_POST_VARS['bcc'] . "\n";
        }
        if (!empty($HTTP_POST_VARS['subject'])) {
            $hdrs .= "Subject: " . $HTTP_POST_VARS['subject'] . "\n";
        }
        if (isset($mime)) {
            $env = $mime->header();
            foreach ($env as $key => $val) {
                $hdrs .= $key . ': ' . $val . "\n";
            }
        }
        $hdrs .= "\n";
        
        $body = $hdrs . $body;
        $body = str_replace("\n", "\r\n", $body);
        
        if (isset($append_flags)) {
            $append_flags .= ' \\Seen';
        } else {
            $append_flags = '\\Seen';
        }
        IMP_folder::create($imp['stream'], preambleString() . $drafts_folder, $prefs->getValue('subscribe'));
        if (!(@imap_append($imp['stream'], serverString() . imap_utf7_encode(preambleString() . $drafts_folder), $body, $append_flags))) {
            echo "<script language=\"JavaScript\" type=\"text/javascript\">alert('Message save failed');</script>";
        } else {
            if ($prefs->getValue('compose_popup') && $browser->hasFeature('javascript')) {
                echo '<script language="JavaScript" type="text/javascript">window.close();</script>';
            } else {
                header('Location: ' . Horde::url(mailboxReturnURL()));
            }
            exit;
        }
    }
    $get_sig = false;
    $ACTION_TEXT = _("Message Postponed");
    break;
    
case ADD_ATTACHMENT:
    if (isset($HTTP_POST_VARS['attachments_name']) && !is_array($HTTP_POST_VARS['attachments_name'])) {
        $HTTP_POST_VARS['attachments_name'] = array($HTTP_POST_VARS['attachments_name']);
    }
    if (isset($HTTP_POST_VARS['attachments_size']) && !is_array($HTTP_POST_VARS['attachments_size'])) {
        $HTTP_POST_VARS['attachments_size'] = array($HTTP_POST_VARS['attachments_size']);
    }
    if (isset($HTTP_POST_VARS['attachments_file']) && !is_array($HTTP_POST_VARS['attachments_file'])) {
        $HTTP_POST_VARS['attachments_file'] = array($HTTP_POST_VARS['attachments_file']);
    }
    if (isset($HTTP_POST_VARS['attachments_type']) && !is_array($HTTP_POST_VARS['attachments_type'])) {
        $HTTP_POST_VARS['attachments_type'] = array($HTTP_POST_VARS['attachments_type']);
    }
    
    // if the filename isn't in the upload directory, ignore the attachment
    if ($HTTP_POST_FILES['file_upload']['size'] > 0 && (tmpdir() == dirname($HTTP_POST_FILES['file_upload']['tmp_name']))) {
        $safe_file = safe_file($HTTP_POST_FILES['file_upload']['tmp_name']);
        $HTTP_POST_VARS['attachments_name'][] = $HTTP_POST_FILES['file_upload']['name'];
        $HTTP_POST_VARS['attachments_size'][] = $HTTP_POST_FILES['file_upload']['size'];
        $HTTP_POST_VARS['attachments_file'][] = $safe_file . '.att';
        $HTTP_POST_VARS['attachments_type'][] = (isset($HTTP_POST_FILES['file_upload']['type'])) ? $HTTP_POST_FILES['file_upload']['type'] : '';
        copy($safe_file, $safe_file . '.att');
        chmod($safe_file . '.att', 0600);
    }
    
    $ACTION_TEXT = _("Composing: Added an attachment...");
    $msg = $HTTP_POST_VARS['message'];
    $get_sig = false;
    break;

case DELETE_ATTACHMENT:
    if (isset($HTTP_POST_VARS['attachments_name']) && !is_array($HTTP_POST_VARS['attachments_name'])) {
        $HTTP_POST_VARS['attachments_name'] = array($HTTP_POST_VARS['attachments_name']);
    }
    if (isset($HTTP_POST_VARS['attachments_size']) && !is_array($HTTP_POST_VARS['attachments_size'])) {
        $HTTP_POST_VARS['attachments_size'] = array($HTTP_POST_VARS['attachments_size']);
    }
    if (isset($HTTP_POST_VARS['attachments_file']) && !is_array($HTTP_POST_VARS['attachments_file'])) {
        $HTTP_POST_VARS['attachments_file'] = array($HTTP_POST_VARS['attachments_file']);
    }
    if (isset($HTTP_POST_VARS['attachments_type']) && !is_array($HTTP_POST_VARS['attachments_type'])) {
        $HTTP_POST_VARS['attachments_type'] = array($HTTP_POST_VARS['attachments_type']);
    }
    if (isset($HTTP_POST_VARS['delattachments']) && !is_array($HTTP_POST_VARS['delattachments'])) {
        $HTTP_POST_VARS['delattachments'] = array($HTTP_POST_VARS['delattachments']);
    }

    if (isset($HTTP_POST_VARS['delattachments'])) {
        for ($i=0; $i<count($HTTP_POST_VARS['delattachments']); $i++) {
            for ($j=0; $j<count($HTTP_POST_VARS['attachments_file']); $j++) {
                if (strcmp($HTTP_POST_VARS['delattachments'][$i], $HTTP_POST_VARS['attachments_file'][$j]) == 0) {
                    if (!preg_match('|.mime$|', $HTTP_POST_VARS['attachments_file'][$j])) {
                        unlink(safe_file($HTTP_POST_VARS['attachments_file'][$j]));
                    }
                    array_splice($HTTP_POST_VARS['attachments_name'], $j, 1);
                    array_splice($HTTP_POST_VARS['attachments_size'], $j, 1);
                    array_splice($HTTP_POST_VARS['attachments_file'], $j, 1);
                    array_splice($HTTP_POST_VARS['attachments_type'], $j, 1);
                }
            }
        }
    }

    $msg = $HTTP_POST_VARS['message'];
    $ACTION_TEXT = _("Composing: Deleted an attachment...");
    $get_sig = false;
    break;

case BOUNCE_COMPOSE:
    $ACTION_TEXT = _("Bounce this message");
    break;
    
case SPELL_CHECK_CANCEL:
    $msg = "\n" . $HTTP_POST_VARS['oldmsg'];
    $get_sig = false;
    break;

case SPELL_CHECK_DONE:
    $msg = "\n";
    if (isset($HTTP_POST_VARS['newmsg'])) $msg .= $HTTP_POST_VARS['newmsg'];
    $msg .= $HTTP_POST_VARS['message'];
    $get_sig = false;
    break;

case SPELL_CHECK:
case SPELL_CHECK_FORWARD:
    include './spelling.php';
    break;

default:
    $ACTION_TEXT = _("Compose a message");
    break;
}

if (!isset($ACTION_TEXT)) $ACTION_TEXT = _("Compose a message");

/* Set the 'save_sent_mail' checkbox for the form. */
$ssm_check = ($HTTP_POST_VARS['save_sent_mail']) ? ' checked="checked"' : '';

$title = _("Message Composition");
require $conf['paths']['templates'] . '/doctype.inc';
require $conf['paths']['templates'] . '/common-header.inc';

if ($prefs->getValue('compose_popup')) {
    $cancel_js = 'confirmCancel();';
} else {
    $cancel_js = 'window.location = \'' . Horde::url(mailboxReturnURL()) . '\';';
    require './menu.php';
}

$barefrom = get_barefrom();
$from = get_from();

/* Grab any data that we were supplied with. */
if (empty($msg)) $msg = Horde::getFormData('msg', '');
if (empty($to)) $to = Horde::getFormData('to', '');
if (empty($cc)) $cc = Horde::getFormData('cc', '');
if (empty($bcc)) $bcc = Horde::getFormData('bcc', '');
if (empty($subject)) $subject = Horde::getFormData('subject');

if ($get_sig && ($sig = $prefs->getValue('signature'))) {
    if ($prefs->getValue('sig_dashes')) {
        $sig = "-- \n$sig";
    }
    if (isset($msg) && $prefs->getValue('sig_first')) {
        $msg = "\n\n\n" . $sig . "\n\n\n" . $msg;
    } else {
        $msg .= "\n\n\n" . $sig;
    }
}

if ($browser->hasFeature('javascript')) {
    include $conf['paths']['templates'] . '/compose/javascript.inc';
}

if (isset($actionID) && ($actionID == BOUNCE_COMPOSE)) {
    include $conf['paths']['templates'] . '/compose/bounce.inc';
} else if (isset($actionID) && ($actionID == SPELL_CHECK || $actionID == SPELL_CHECK_FORWARD)) {
    include $conf['paths']['templates'] . '/compose/spelling.inc';
} else {
    include $conf['paths']['templates'] . '/compose/compose.inc';
}

if ($conf['user']['online_help'] && $browser->hasFeature('javascript')) {
    Help::javascript();
}
$registry->shutdown();

require $conf['paths']['templates'] . '/common-footer.inc';

$prefs->store();

?>
