<?php
/*
 * $Horde: imp/filters.php,v 2.9 2001/01/10 19:24:21 chuck Exp $
 *
 * Copyright 2000 Charles J. Hagenbuch <chuck@horde.org>
 * Copyright 2000 Max Kalika <max@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

require_once './lib/base.php';
require_once '../lib/Registry.php';

$registry = new Registry();

/* retrieve authentication information from session */
session_name($horde['session_name']);
@session_start();

if (!isset($HTTP_SESSION_VARS['imp']) || !is_array($HTTP_SESSION_VARS['imp'])) {
    header('Location: ' . Horde::url('login.php?reason=logout', true));
    exit;
}
IMP::setupSession();
IMP::authenticate(OP_HALFOPEN);

/* redirect back to the mailbox if folder use is not allowed */
if (!($conf['user']['allow_folders'])) {
    header('Location: ' . Horde::url('mailbox.php?', true));
    exit;
}

$preamble = preambleString();
$errormsg = '';
$actionID = IMP_util::getAction();

/* Run through the action handlers */
if (isset($actionID)) {
    switch ($actionID) {
        
    case NO_ACTION:
        break;
        
    case FILTER_CREATE:
        if (isset($HTTP_POST_VARS['field']) && isset($HTTP_POST_VARS['text']) && isset($HTTP_POST_VARS['action']) && !preg_match('/^\s*$/', $HTTP_POST_VARS['text'])) {
            $filters = @unserialize($prefs->getValue('filters'));
            if ($HTTP_POST_VARS['action'] == 'delete') {
                $filters[] = array('field' => $HTTP_POST_VARS['field'], 'text' => chop(trim($HTTP_POST_VARS['text'])), 'action' => 'delete');
            } elseif ($HTTP_POST_VARS['action'] == 'move' && isset($HTTP_POST_VARS['folder']) && $HTTP_POST_VARS['folder']) {
                $filters[] = array('field' => $HTTP_POST_VARS['field'], 'text' => chop(trim($HTTP_POST_VARS['text'])), 'action' => 'move', 'folder' => $HTTP_POST_VARS['folder']);
            } else {
                Horde::raiseMessage(_("no folder selected"), HORDE_ERROR);
                $errorRaised = true;
            }

            $prefs->setValue('filters', serialize($filters));
            $prefs->store();
        }
        else {
            Horde::raiseMessage(_("please fill in the text and choose a field and an action"), HORDE_ERROR);
            $errorRaised = true;
        }
        break;

    case FILTER_MODIFY:
        if (isset($HTTP_POST_VARS['number'])) {
            if (isset($HTTP_POST_VARS['text']) && !preg_match('/^\s*$/', $HTTP_POST_VARS['text'])) {
                $filters = @unserialize($prefs->getValue('filters'));
                if ($action == 'delete') {
                    $filters[$HTTP_POST_VARS['number']] = array('field' => $HTTP_POST_VARS['field'], 'text' => chop(trim($HTTP_POST_VARS['text'])), 'action' => 'delete');
                } elseif ($action == 'move' && isset($HTTP_POST_VARS['folder']) && $HTTP_POST_VARS['folder']) {
                    $filters[$HTTP_POST_VARS['number']] = array('field' => $HTTP_POST_VARS['field'], 'text' => chop(trim($HTTP_POST_VARS['text'])), 'action' => 'move', 'folder' => $HTTP_POST_VARS['folder']);
                } else {
                    Horde::raiseMessage(_("no folder selected"), HORDE_ERROR);
                    $errorRaised = true;
                }

                $prefs->setValue('filters', serialize($filters));
                $prefs->store();
            } else {
                Horde::raiseMessage(_("no text specified"), HORDE_ERROR);
                $errorRaised = true;
            }
        } else {
            Horde::raiseMessage(_("no rule selected"), HORDE_ERROR);
        }
        break;

    case FILTER_DELETE:
        if (isset($HTTP_POST_VARS['number'])) {
            $filters = @unserialize($prefs->getValue('filters'));
            array_splice($filters, $HTTP_POST_VARS['number'], 1);
            $prefs->setValue('filters', serialize($filters));
            $prefs->store();
        } else {
            Horde::raiseMessage(_("no rule selected"), HORDE_ERROR);
        }
        break;

    case FILTER_DOWN:
        if (isset($HTTP_POST_VARS['number'])) {
            $filters = @unserialize($prefs->getValue('filters'));
            if($HTTP_POST_VARS['number'] < count($filters) - 1) {
                $tmpFilter = array_splice($filters, $HTTP_POST_VARS['number'], 1);
                array_splice($filters, $HTTP_POST_VARS['number'] + 1, 0, $tmpFilter);
                $prefs->setValue('filters', serialize($filters));
                $prefs->store();
            } else {
                Horde::raiseMessage(_("cannot move past the end"), HORDE_WARNING);
                $errorRaised = true;
            }
        } else {
            Horde::raiseMessage(_("no rule selected"), HORDE_ERROR);
        }
        break;

    case FILTER_UP:
        if (isset($HTTP_POST_VARS['number'])) {
            $filters = @unserialize($prefs->getValue('filters'));
            if($HTTP_POST_VARS['number']) {
                $tmpFilter = array_splice($filters, $HTTP_POST_VARS['number'], 1);
                array_splice($filters, $HTTP_POST_VARS['number'] - 1, 0, $tmpFilter);
                $prefs->setValue('filters', serialize($filters));
                $prefs->store();
            } else {
                Horde::raiseMessage(_("cannot move past the beginning"), HORDE_WARNING);
                $errorRaised = true;
            }
        } else {
            Horde::raiseMessage(_("no rule selected"), HORDE_ERROR);
        }
        break;

    } // switch
} // isset($actionID)

$title = _("Mail Filters");

require $conf['paths']['templates'] . '/doctype.inc';
require $conf['paths']['templates'] . '/common-header.inc';
require './menu.php';

$filters = @unserialize($prefs->getValue('filters'));
$apply_js = 'window.location = \'' . Horde::url('mailbox.php?actionID=' . FILTER, true) . '\';';
if (count(IMP::flist(array('INBOX')))) {
    $options = IMP::flistSelect(_("select folder"), true, array('INBOX'));
}

$fieldsList['to'] = 0;
$fieldsList['cc'] = 1;
$fieldsList['from'] = 2;
$fieldsList['subject'] = 3;

$actionsList['delete'] = 0;
$actionsList['move'] = 1;

require $conf['paths']['templates'] . '/filters/javascript.inc';
require $conf['paths']['templates'] . '/filters/list.inc';
require $conf['paths']['templates'] . '/filters/manage.inc';


if ($conf['user']['online_help'] && $browser->hasFeature('javascript')) {
    Help::javascript();
}
$registry->shutdown();

require $conf['paths']['templates'] . '/common-footer.inc';

$prefs->store();

?>
