<?php
/*
 * $Horde: imp/folders.php,v 2.96 2001/01/10 19:24:21 chuck Exp $
 *
 * Copyright 2000 Charles J. Hagenbuch <chuck@horde.org>
 * Copyright 2000 Jon Parise <jon@horde.org>
 * Copyright 2000 Anil Madhavapeddy <avsm@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

define('NAV_REFRESH_NEVER', -2);
define('NAV_REFRESH_NOW', -1);

require_once './lib/base.php';
require_once './lib/tree.php';
require_once '../lib/Registry.php';
require_once './lib/folder.php';

$registry = new Registry();

/* Redirect back to the mailbox if folder use is not allowed. */
if (!$conf['user']['allow_folders']) {
    header('Location: ' . Horde::url('mailbox.php?', true));
    exit;
}

/* Retrieve authentication information from the session. */
session_name($horde['session_name']);
@session_start();

if ((!isset($HTTP_SESSION_VARS['imp'])) || (!is_array($HTTP_SESSION_VARS['imp']))) {
    header('Location: ' . Horde::url('login.php?reason=logout', true));
    exit;
}

IMP::setupSession();
IMP::authenticate(OP_HALFOPEN);

$imaptree = new IMP_tree();

/* Check to see if user has a defined set of folders to poll */
if ($prefs->isChangeable('nav_poll')) {
    $folderPollList = @unserialize($prefs->getValue('nav_poll'));
} 

/* If not, then only check the INBOX */
if (!isset($folderPollList) || !is_array($folderPollList)) {
    $folderPollList = array('INBOX' => true);
}

/* This reinitializes the $imaptree variable and recalculates the
 * folder list */
function initializeTree() {
    global $prefs, $imaptree, $showAll;
    $imaptree->init($prefs->getValue('subscribe') && !$showAll);
    if ($prefs->getValue('nav_expanded')) {
        $imaptree->expandAllFolders();
    }
}

/* Utility function to return a url for the various folder images */
function folderImage($name, $alt, $width , $height ) {
    global $conf;
    return '<img src="' . $conf['paths']['graphics'] . '/folders/' . $name .
        '" hspace="3" border="0" alt="' . $alt . '" width="'.$width.
        '" height="'.$height.'" />';
}

/* Decide whether or not to show all the unsubscribed folders */
if ($prefs->getValue('subscribe') && isset($imp['showunsub'])) {
    $showAll = $imp['showunsub'];
} else {
    $showAll = false;
}

if (isset($imp['imaptree']) && is_array($imp['imaptree'])) {
    $imaptree->unpickle($imp['imaptree']);
} else {
   initializeTree();
}

$refresh_time = $prefs->getValue('refresh_time');
 
/* Run through the action handlers */
$actionID = IMP_util::getAction();
switch ($actionID) {
    
 case NO_ACTION:
     break;
     
 case EXPAND_FOLDER:
     if (!empty($HTTP_GET_VARS['folder'])) {
         $imaptree->expand($HTTP_GET_VARS['folder']);
     }
     break;
     
 case EXPAND_ALL_FOLDERS:
     $imaptree->expandAllFolders();
     break;
     
 case COLLAPSE_FOLDER:
     if (!empty($HTTP_GET_VARS['folder'])) {
         $imaptree->collapse($HTTP_GET_VARS['folder']);
     }
     break;
     
 case COLLAPSE_ALL_FOLDERS:
     $imaptree->collapseAllFolders();
     break;
     
 case REFRESH_FOLDERS:
     initializeTree();
     break;
     
 case DELETE_FOLDER:
     if (is_array($HTTP_POST_VARS['folder_list'])) {
         if (IMP_folder::delete($imp['stream'], $HTTP_POST_VARS['folder_list'], $prefs->getValue('subscribe'))) {
             foreach($HTTP_POST_VARS['folder_list'] as $folder) {
                 $imaptree->delete(serverString() . $folder);
             }
         }
     }
     break;
     
 case CREATE_FOLDER:
     if (!empty($HTTP_POST_VARS['new_mailbox'])) {
         if (isset($HTTP_POST_VARS['folder_list']) &&
             is_array($HTTP_POST_VARS['folder_list']) &&
             count($HTTP_POST_VARS['folder_list']) == 1) {
             $new_mailbox = $HTTP_POST_VARS['folder_list'][0] . $imaptree->delimiter . $HTTP_POST_VARS['new_mailbox'];
         } elseif (!empty($imp['folders'])) {
             $new_mailbox = $imp['folders'] . $HTTP_POST_VARS['new_mailbox'];
         } else {
             $new_mailbox = $HTTP_POST_VARS['new_mailbox'];
         }
         
         if (IMP_folder::create($imp['stream'], $new_mailbox, $prefs->getValue('subscribe'))) {
             initializeTree();
         }
     }
     break;
     
 case RENAME_FOLDER:
     $new_names = explode("\n", $HTTP_POST_VARS['new_names']);
     $old_names = explode("\n", $HTTP_POST_VARS['old_names']);
     if (!empty($new_names) && !empty($old_names) && 
         (count($new_names) == count($old_names))) {
         for ($i = 0; $i < sizeof($new_names); $i++) {
             IMP_folder::rename($imp['stream'], trim($old_names[$i]),
                                preambleString() . trim($new_names[$i]), $prefs->getValue('subscribe'));
         }
         initializeTree();
     }
     break;
     
 case SUBSCRIBE_FOLDER:
     if (isset($HTTP_POST_VARS['folder_list']) && is_array($HTTP_POST_VARS['folder_list'])) {
         IMP_folder::subscribe($imp['stream'], $HTTP_POST_VARS['folder_list']);
     } else {
         Horde::raiseMessage(_('No folders were specified'), HORDE_MESSAGE);
     }
     initializeTree();
     break;
     
 case UNSUBSCRIBE_FOLDER:
     if (isset($HTTP_POST_VARS['folder_list']) && is_array($HTTP_POST_VARS['folder_list'])) {
         IMP_folder::unsubscribe($imp['stream'], $HTTP_POST_VARS['folder_list']);
     } else {
         Horde::raiseMessage(_('No folders were specified'), HORDE_MESSAGE);
     }
     initializeTree();
     break;
     
 case TOGGLE_SUBSCRIBED_VIEW: 
     if ($prefs->getValue('subscribe')) {
         $imp['showunsub'] = ($showAll = !$showAll);
     }
     initializeTree();
     break;        
     
 case POLL_FOLDER:
     if (isset($HTTP_POST_VARS['folder_list']) &&
         is_array($HTTP_POST_VARS['folder_list']) &&
         $prefs->isChangeable('nav_poll')) {
         
         foreach ($HTTP_POST_VARS['folder_list'] as $folder) {
             $folderPollList[$folder] = true;
         }
         $prefs->setValue('nav_poll', serialize($folderPollList));
         $prefs->store();
     }
     break;
     
 case NOPOLL_FOLDER:
     if (isset($HTTP_POST_VARS['folder_list']) &&
         is_array($HTTP_POST_VARS['folder_list']) &&
         $prefs->isChangeable('nav_poll')) {
         
         foreach ($HTTP_POST_VARS['folder_list'] as $folder) {
             if (isset($folderPollList[$folder])) {
                 unset($folderPollList[$folder]);
             }
         }
         $prefs->setValue('nav_poll', serialize($folderPollList));
         $prefs->store();
     }
     break;
}

/* Display the correct message on the action bar */
$subToggleText = $showAll?_('Hide Unsubscribed'):_('Show Unsubscribed');

/* Set the URL to refresh the page too in the META tag */
$refresh_url = Horde::url('folders.php', true);

if ($conf['compress_pages']) {
    HTTP_Cache::start();
}

$title = _('Folder Navigator');
require $conf['paths']['templates'] . '/doctype.inc';
require $conf['paths']['templates'] . '/common-header.inc';
require './menu.php';

/* Default to the very top of the hierarchy */
if (!isset($pattern)) {
    $pattern = '%';
}

$i = 0;
$imaptree->reset();
$newmsgs = array();
$rowct = false;

require $conf['paths']['templates'] . '/folders/javascript.inc';
require $conf['paths']['templates'] . '/folders/head.inc';
require $conf['paths']['templates'] . '/folders/actions.inc';

/* Always poll the INBOX for new mail */
$nm = $imaptree->updateMessageInfo();
$hd = $imaptree->head();
if (isset($hd['newmsg']) && $hd['newmsg'] > 0) {
    $newmsgs['INBOX'] = $hd['newmsg'];
}

if ($showAll) {
    $unsubList = IMP::flistUnsubscribed();
}

/* Start iterating through the list of mailboxes, displaying them */
$mailbox = $imaptree->head();

while (isset($mailbox) && is_array($mailbox)) {

    /* Determine if we need to poll this mailbox for new messages */
    if (isset($folderPollList[$mailbox['value']])) {
        $updateCache = true;
    } else {
        $updateCache = false;
    }

    /* If we have stale information in the imaptree cache, then zero it */
    if (!$updateCache && isset($mailbox['messages'])) {
        $imaptree->flushMessageInfo();
        $mailbox = $imaptree->current();
    }

    /* If we need message information for this folder, update it now */
    if ($updateCache) { 
        $imaptree->updateMessageInfo();
        $mailbox = $imaptree->current();
    }

    /* Highlight mailboxes with unread messages in bold */
    if (isset($mailbox['unseen']) && $mailbox['unseen'] > 0) {
        $mailbox['label'] = '<b>' . $mailbox['label'] . '</b>';
    }

    /* Populate the $newmsgs hash with new msgs since last view of the navigator */
    if (isset($mailbox['newmsg']) && $mailbox['newmsg'] > 0) {
        $newmsgs[$mailbox['value']] = $mailbox['newmsg'];
    }

    if (!($mailbox['attributes'] & LATT_NOSELECT)) {
        $name = Horde::link(Horde::url('mailbox.php?mailbox=' . urlencode($mailbox['value'])),
                _('View messages in ') . IMP_folder::displayName($mailbox['value'])) . 
                $mailbox['label'] . '</a>';

        if ($mailbox['label'] == 'INBOX') {
            $dir2 = folderImage('inbox.gif', _('INBOX'), 18, 18);
        } elseif ($mailbox['attributes'] & LATT_MARKED) {
            $dir2 = folderImage('mbox_marked.gif', _('Mailbox'), 16, 16);
        } elseif ($mailbox['attributes'] & LATT_UNMARKED) {
            $dir2 = folderImage('mbox_unmarked.gif', _('Mailbox'), 16, 16);
        } else {
            $dir2 = folderImage('mbox.gif', _('Mailbox'), 16, 16);
        }
    } else {
        $name = $mailbox['label'];
        if ($imaptree->isOpen($mailbox)) {
            $dir2 = folderImage('folder_open.gif', _('Open Folder'), 16, 16);
        } else {
            $dir2 = folderImage('folder.gif', _('Closed Folder'), 16, 16);
        }
    }
    
    if ($imaptree->hasChildren($mailbox)) {
        
        if ($imaptree->isOpen($mailbox)) {
            $dir = Horde::link(Horde::url('folders.php?actionID=' . COLLAPSE_FOLDER . "&folder=" .
                   urlencode($imaptree->server . $mailbox['value'])), _('Collapse Folder')) .
                   folderImage('expanded.gif', _('Collapse Folder'), 16, 16) . "</a>$dir2";
        } else {
            $dir = Horde::link(Horde::url('folders.php?actionID=' . EXPAND_FOLDER . "&folder=" .
                   urlencode($imaptree->server . $mailbox['value'])), _('Expand Folder')) .
                   folderImage('collapsed.gif', _('Expand Folder'), 16, 16) . "</a>$dir2";
        }
    } else {
        $dir = folderImage('empty.gif','', 16, 16) . $dir2;
    }
  
    /* set up status information */
    if ($name == $mailbox['label']) {
        $msgs = '&nbsp;';
        $new  = '&nbsp;';
    } else {
        if (isset($mailbox['messages'])) {
            $msgs = $mailbox['messages'];
        } else {
            $msgs = '&nbsp;';
        }
        if (isset($mailbox['unseen'])) {
            $new = $mailbox['unseen'];
        } else {
            $new = '&nbsp;';
        }
    }
    
    if ($showAll && isset($unsubList[$mailbox['value']])) {
        $navclass = 'navigatorunsub';
    } else {
        $navclass = ($rowct = !$rowct) ? 'navigator' : 'navigatorlt'; 
    }

    include $conf['paths']['templates'] . '/folders/row.inc';
    $i++;
    
    if ($imaptree->isOpen($mailbox)) {
        $mailbox = $imaptree->next();
    } else {
        $mailbox = $imaptree->nextOnLevel();
    }
}

/* Put the tree back into the session object for caching */
$imp['imaptree'] = $imaptree->pickle();

require $conf['paths']['templates'] . '/folders/actions.inc';
require $conf['paths']['templates'] . '/folders/foot.inc';

/* Check to see if user wants new mail notification */
if ($prefs->getValue('nav_popup')) {
    $alert = '';
    while (list($mb, $nm) = each($newmsgs)) {
        $alert .= IMP_folder::displayName($mb) . _(" - $nm new message" . ($nm > 1 ? 's' : '')) . '\n';
    }
    if (!empty($alert)) {
        include $conf['paths']['templates'] . '/folders/alert.inc';
    }
}

if ($conf['user']['online_help'] && $browser->hasFeature('javascript')) {
    Help::javascript();
}
$registry->shutdown();

require $conf['paths']['templates'] . '/common-footer.inc';

if ($conf['compress_pages']) {
    HTTP_Cache::output();
}

?>
