<?php
// $Horde: imp/lib/IMP.php,v 1.117 2001/01/08 22:52:39 chuck Exp $

define('IMP_NAME', 'Internet Messaging Program (IMP)');

require_once './lib/version.php';
require_once '../lib/Prefs.php';

// define IMP actions
define('NO_ACTION',               0);
define('DELETE_MESSAGES',         1);
define('UNDELETE_MESSAGES',       2);
define('MOVE_MESSAGES',           3);
define('COPY_MESSAGES',           4);
define('EXPUNGE_MAILBOX',         5);
define('IMP_LOGIN',               6);
define('REPLY',                   7);
define('REPLY_ALL',               8);
define('FORWARD',                 9);
define('DELETE_FOLDER',          10);
define('CREATE_FOLDER',          11);
define('RENAME_FOLDER',          12);
define('DOWNLOAD_ATTACH',        13);
define('VIEW_ATTACH',            14);
define('DELETE_ADDR',            15);
define('ADD_ADDR',               16);
define('UPDATE_ADDR',            17);
define('UPDATE_PREFS',           18);
define('SEND_MESSAGE',           19);
define('POSTPONE_MESSAGE',       20);
define('VIEW_SOURCE',            21);
define('SAVE_MESSAGE',           22);
define('SUBSCRIBE_FOLDER',       23);
define('UNSUBSCRIBE_FOLDER',     24);
define('BOUNCE_MESSAGE',         25);
define('BOUNCE_COMPOSE',         26);
define('SPAM_REPORT',            27);
define('ADD_ATTACHMENT',         28);
define('DELETE_ATTACHMENT',      29);
define('MAILTO',                 30);
define('ADD_CONTACT',            31);
define('IMP_SIGNUP',             32);
define('COMPOSE',                33);
define('DRAFT',                  34);
define('SAVE_DRAFT',             35);
define('HIDE_DELETED',           36);
define('LOGIN_COMPOSE',          37);
define('SEARCH',                 38);
define('SPELL_CHECK',            39);
define('SPELL_CHECK_FORWARD',    40);
define('SPELL_CHECK_CANCEL',     41);
define('SPELL_CHECK_DONE',       42);
define('RETURN_TO_MAILBOX',      43);
define('PASSWD_INPUT',           44);
define('PASSWD_DISALLOW',        45);
define('PASSWD_CHANGE',          46);
define('EXPAND_FOLDER',          47);
define('COLLAPSE_FOLDER',        48);
define('EXPAND_ALL_FOLDERS',     49);
define('COLLAPSE_ALL_FOLDERS',   50);
define('REFRESH_FOLDERS',        51);
define('POLL_FOLDER',            52);
define('NOPOLL_FOLDER',          53);
define('TOGGLE_SUBSCRIBED_VIEW', 54);
define('FLAG_MESSAGES',          55);
define('PRINT_MESSAGE',          56);
define('FILTER',                 57);
define('FILTER_CREATE',          58);
define('FILTER_MODIFY',          59);
define('FILTER_DELETE',          60);
define('FILTER_DOWN',            61);
define('FILTER_UP',              62);

// defines for IMAP flags
define('IMP_ALL',                 0);
define('IMP_UNSEEN',              1);
define('IMP_DELETED',             2);
define('IMP_ANSWERED',            4);
define('IMP_FLAGGED',             8);
define('IMP_DRAFT',              16);
define('IMP_PERSONAL',           32);


/* Set the umask. */
if (isset($conf['umask'])) {
    umask($conf['umask']);
}


class IMP {
    
    /**
     * Take posted from the login screen and attempt to set up an
     * initial IMP session. This function should only be called once,
     * when the user first logs in.
     *
     * @access public
     */
    function createSession()
    {
        global $imp, $conf, $prefs, $HTTP_POST_VARS;
        
        if (!isset($HTTP_POST_VARS['imapuser'])
            || !isset($HTTP_POST_VARS['pass'])
            || !isset($HTTP_POST_VARS['server'])) {
            return false;
        }
        
        $imp = array();
        
        $imp['user'] = trim($HTTP_POST_VARS['imapuser']);
        $imp['pass'] = Secret::write(IMP::setKey(), trim($HTTP_POST_VARS['pass']));
        
        /* TODO: reimplement this using the new Prefs system. */
        // $imp['prefs'] = array();
        
        /* Create a new preferences object and load the site defaults. */
        $prefs = Prefs::factory($conf['prefs']['driver'], $imp['user'],
                                Secret::read(IMP::getKey(), $imp['pass']), $conf['prefs']['params']);
        $prefs->setDefaults('./config/prefs.php');
        
        /* We might need to override some of the defaults with
           environmental settings. */
        if ($conf['server']['server_list']) {
            include_once './config/servers.php';
        }
        
        if ($conf['server']['server_list']
            && isset($servers[$HTTP_POST_VARS['server']])
            && is_array($servers[$HTTP_POST_VARS['server']])) {
            $svr = &$HTTP_POST_VARS['server'];
            $prefs->setValue('mailhost', $servers[$svr]['server']);
            $prefs->setValue('mailport', $servers[$svr]['port']);
            $prefs->setValue('mailprotocol', $servers[$svr]['protocol']);
            $prefs->setValue('maildomain', $servers[$svr]['to_host']);
            $prefs->setValue('folders', $servers[$svr]['folders']);
        } else {
            if ($prefs->isChangeable('mailhost') && !empty($HTTP_POST_VARS['server']))
                $prefs->setValue('mailhost', $HTTP_POST_VARS['server']);
            
            if ($prefs->isChangeable('mailport') && !empty($HTTP_POST_VARS['port']))
                $prefs->setValue('mailport', $HTTP_POST_VARS['port']);
            
            if ($prefs->isChangeable('mailprotocol') && !empty($HTTP_POST_VARS['protocol']))
                $prefs->setValue('mailprotocol', $HTTP_POST_VARS['protocol']);
            
            if ($prefs->isChangeable('maildomain') && !empty($HTTP_POST_VARS['maildomain']))
                $prefs->setValue('maildomain', $HTTP_POST_VARS['maildomain']);
            
            if ($prefs->isChangeable('folders') && !empty($HTTP_POST_VARS['folders']))
                $prefs->setValue('folders', $HTTP_POST_VARS['folders']);
        }
        
        /* Attempt to retrieve the preferences from storage. */
        $prefs->retrieve();
        
        /* Set the session variables.  These are cached. */
        $imp['server'] = $prefs->getValue('mailhost');
        $imp['port'] = $prefs->getValue('mailport');
        $imp['folders'] = $prefs->getValue('folders');
        $imp['protocol'] = $prefs->getValue('mailprotocol');
        $imp['maildomain'] = $prefs->getValue('maildomain');
        $imp['mailbox'] = $prefs->getValue('mailbox');
        IMP::setLabel($imp);
        
        /* Support the special value '@@' for maildomain. */
        if ($imp['maildomain'] == '@@') {
            $imp['maildomain'] = $imp['server'];
        }
        
        $GLOBALS['HTTP_SESSION_VARS']['imp'] = &$imp;
        IMP::setupSession();
        session_register('imp');
        
        return true;
    }
    
    /**
     * Set up the session for use in the page. Retrieve preferences,
     * and make sure to update the mailbox, the sorting information,
     * etc, if any of them have been changed. Also does
     * protocol-specific tasks, such as automatically disabling
     * folders if pop3 is being used.
     *
     * @access public
     */
    function setupSession()
    {
        global $conf, $prefs, $HTTP_GET_VARS;
        
        if (!isset($GLOBALS['HTTP_SESSION_VARS']['imp'])) {
            return false;
        } elseif (!isset($GLOBALS['imp'])) {
            $GLOBALS['imp'] = &$GLOBALS['HTTP_SESSION_VARS']['imp'];
        }
        
        if (!isset($prefs)) {
            $prefs = Prefs::factory($conf['prefs']['driver'], $GLOBALS['imp']['user'],
                                    Secret::read(IMP::getKey(), $GLOBALS['imp']['pass']), $conf['prefs']['params']);
            $prefs->setDefaults('./config/prefs.php');
        }
        $prefs->retrieve();
        
        if (isset($HTTP_GET_VARS['sortby'])) {
            $prefs->setValue('sortby', $HTTP_GET_VARS['sortby']);
        }
        if (isset($HTTP_GET_VARS['sortdir'])) {
            $prefs->setValue('sortdir', $HTTP_GET_VARS['sortdir']);
        }
        
        if (!empty($HTTP_GET_VARS['mailbox'])) {
            $GLOBALS['imp']['mailbox'] = $HTTP_GET_VARS['mailbox'];
            IMP::setLabel($GLOBALS['imp']);
        }
        
        switch ($GLOBALS['imp']['protocol']) {
        case 'pop3':
            $conf['user']['allow_folders'] = false;
            break;
        }
        
        return true;
    }
    
    /**
     * Attempt to open a connection to the IMAP server using the
     * information in the $imp session variable.
     *
     * @access public
     *
     * @param int $flags  (optional) Flags to pass to IMAP_OPEN. Valid values are:
     * <pre>
     *     OP_READONLY  : Open the mailbox read-only.
     *     OP_ANONYMOUS : (NNTP only) Don't use or update a .newrc file.
     *     OP_HALFOPEN  : (IMAP and NNTP only) Open a connection, but not a specific mailbox.
     *     CL_EXPUNGE   : Expunge the mailbox automatically when the stream is closed.
     * </pre>
     *
     * @return bool $ok   Success or failure.
     */
    function authenticate($flags = 0)
    {
        global $imp, $conf, $HTTP_SERVER_VARS, $HTTP_POST_VARS;
        
        if (!(isset($imp) && is_array($imp))) {
            return false;
        }
        
        if (isset($HTTP_POST_VARS['connID']) && $conf['connections']['track']) {
            include_once '../lib/Connection.php';
            $conn = Connection::factory($conf['connections']['driver'], $conf['connections']['params']);
            if (!$conn->verify($HTTP_POST_VARS['connID'])) {
                if ($conf['log']['enabled']) {
                    include_once 'Log.php';
                    $log = &Log::singleton($conf['log']['type'], $conf['log']['name'], $conf['log']['ident'], $conf['log']['conf']);
                    
                    $entry = sprintf('PASSWORD RELOADED %s to %s:%s[%s] as %s',
                                     $HTTP_SERVER_VARS['REMOTE_ADDR'], $imp['server'], $imp['port'], $imp['protocol'], $imp['user']);
                    $log->log($entry, LOG_INFO);
                }
                
                session_unregister('imp');
                return false;
            }
        }
        
        switch ($imp['protocol']) {
        case 'pop3':
            $connstr = serverString('pop3');
            break;
        case 'imap':
        default:
            global $HTTP_GET_VARS, $HTTP_POST_VARS;
            if ($imp['mailbox'] == '**search') {
                if (isset($HTTP_GET_VARS['array_index']) && isset($imp['searchfolders'])) {
                    $tmp = explode(':', $imp['messagefolders']);
                    $connstr = serverString('imap') . imap_utf7_encode($tmp[$HTTP_GET_VARS['array_index']]);
                } elseif (isset($HTTP_POST_VARS['array_index']) && isset($imp['searchfolders'])) {
                    $tmp = explode(':', $imp['messagefolders']);
                    $connstr = serverString('imap') . imap_utf7_encode($tmp[$HTTP_POST_VARS['array_index']]);
                } else {
                    $connstr = serverString('imap');
                    $flags = $flags | OP_HALFOPEN;
                }
            } else {
                $connstr = serverString('imap') . imap_utf7_encode($imp['mailbox']);
            }
            break;
        }
        
        if ($flags == 0) {
            $imp['stream'] = @imap_open($connstr, $imp['user'], Secret::read(IMP::getKey(), $imp['pass']));
        } else {
            $imp['stream'] = @imap_open($connstr, $imp['user'], Secret::read(IMP::getKey(), $imp['pass']), $flags);
        }
        
        if (!$imp['stream']) {
            if (!empty($imp['server']) && !empty($imp['port']) &&
                !empty($imp['protocol']) && !empty($imp['user'])) {
                if ($conf['log']['enabled']) { 
                    include_once 'Log.php';
                    $log = &Log::singleton($conf['log']['type'], $conf['log']['name'],
                                           $conf['log']['ident'], $conf['log']['conf']);
                    if (!empty($HTTP_SERVER_VARS['HTTP_X_FORWARDED_FOR'])) {
                        $entry = sprintf('FAILED LOGIN %s (forwarded for [%s]) to %s:%s[%s] as %s',
                                         $HTTP_SERVER_VARS['REMOTE_ADDR'], $HTTP_SERVER_VARS['HTTP_X_FORWARDED_FOR'], $imp['server'], $imp['port'], $imp['protocol'], $imp['user']);
                    } else {
                        $entry = sprintf('FAILED LOGIN %s to %s:%s[%s] as %s',
                                         $HTTP_SERVER_VARS['REMOTE_ADDR'], $imp['server'], $imp['port'], $imp['protocol'], $imp['user']);
                    }
                    $log->log($entry, LOG_INFO);
                }
            }
            
            session_unregister('imp');
            return false;
        }
        return true;
    }
    
    /**
     * Generate a secret key (for encryption), either using a random
     * md5 string and storing it in a cookie if the user has cookies
     * enabled, or munging some known values if they don't.
     *
     * @access private
     * @return The secret key that has been generated.
     */
    function setKey()
    {
        if (isset($GLOBALS['HTTP_COOKIE_VARS'][session_name()])) {
            $key = md5(uniqid(rand()));
            $GLOBALS['HTTP_COOKIE_VARS']['imp_key'] = $key;
            setcookie('imp_key', $key);
        } else {
            $key = md5(session_id() . $GLOBALS['conf']['sitename']);
        }
        
        return $key;
    }
    
    /**
     * Return a secret key, either from a cookie, or if the cookie
     * isn't there, assume we are using a munged version of a known
     * base value.
     *
     * @access public
     * @return The secret key.
     */
    function getKey()
    {
        if (isset($GLOBALS['HTTP_COOKIE_VARS']['imp_key'])) {
            return $GLOBALS['HTTP_COOKIE_VARS']['imp_key'];
        } else {
            return md5(session_id() . $GLOBALS['conf']['sitename']);
        }
    }
    
    /**
     * Set the plain-text label that is displayed for the current
     * mailbox, replacing '**search' with an appropriate string and
     * removing namespace and folder prefix information from what is
     * shown to the user.
     *
     * @access private
     *
     * @param array $imp  The hash to pull mailbox information from and put
     *                    the label in.
     */
    function setLabel(&$imp)
    {
        global $conf;
        
        if ($imp['mailbox'] == '**search') {
            $imp['label'] = _("Search Results");
        } else {
            $imp['label'] = $imp['mailbox'];
            if (strcmp($imp['folders'], substr($imp['label'], 0, strlen($imp['folders']))) == 0) {
                $imp['label'] = substr($imp['label'], strlen($imp['folders']));
            }
            if (strcmp($conf['server']['namespace'], substr($imp['label'], 0, strlen($conf['server']['namespace']))) == 0) {
                $imp['label'] = substr($imp['label'], strlen($conf['server']['namespace']));
            }
        }
    }
    
    /**
     * Takes an address object, as returned by imap_header(), and
     * formats it as a string.
     *
     * Object Format
     * For the address: John Doe <john_doe@example.com>
     * The object fields are:
     * <pre>
     *  $object->personal = Personal name ("John Doe")
     *  $object->mailbox  = The user's mailbox ("john_doe")
     *  $object->host     = The host the mailbox is on ("example.com")
     * </pre>
     *
     * @access public
     *
     * @param Object $ob      The address object to be turned into a string
     * @param string $filter  (optional) A user@example.com style bare address
     *                        to ignore. If the address matches $filter,
     *                        an empty string will be returned.
     *
     * @return string $address  The formatted address (Example: John Doe
     *                          <john_doe@example.com>)
     */
    function addrObject2String($ob, $filter = '')
    {
        include_once '../lib/MIME.php';
        
        /* If the personal name is set, decode it. */
        $ob->personal = isset($ob->personal) ? MIME::decode($ob->personal) : '';
        
        /*
         * If both the mailbox and the host are empty, return an empty
         * string.  If we just let this case fall through, the call to
         * imap_rfc822_write_address will end up return just a '@', which
         * is undesirable.
         */
        if (empty($ob->mailbox) && empty($ob->host)) {
            return '';
        }
        
        /* Make sure these two variables have some sort of value. */
        if (!isset($ob->mailbox)) $ob->mailbox = '';
        if (!isset($ob->host)) $ob->host = '';
        
        /* Filter out unwanted addresses based on the $filter string. */
        if (strcasecmp($filter, $ob->mailbox . '@' . $ob->host) == 0) {
            return '';
        }
        
        /* Return the trimmed, formatted email address. */
        return Mime::trimEmailAddress(imap_rfc822_write_address($ob->mailbox,
                                                                $ob->host, $ob->personal));
    }
    
    /**
     * Takes an array of address objects, as returned by
     * imap_header(), and passes each of them through
     * IMP::addrObject2String().
     *
     * @access public
     *
     * @param array $addresses   The array of address objects.
     * @param string $filter     (optional) A user@example.com style bare address
     *                           to ignore. If any address matches $filter,
     *                           it will not be included in the final string.
     *
     * @return string $addresses  All of the addresses in a comma-delimited
     *                            string.
     */
    function addrArray2String($addresses, $filter = '')
    {
        if (!is_array($addresses)) return '';
        
        foreach ($addresses as $addr) {
            $val = IMP::addrObject2String($addr, $filter);
            if (!empty($val)) $return[] = $val;
        }
        
        if (isset($return) && is_array($return)) {
            return implode(', ', array_unique($return));
        } else {
            return '';
        }
    }
    
    /**
     * Return an array of folders. This is a wrapper around the
     * IMP_folder::flist() function which reduces the number of
     * arguments needed if we can assume that IMP's full environment
     * is present.
     *
     * @access public
     *
     * @param string $filter (optional) A string specifying a mailbox to
     *                       ignore.
     *
     * @return array $mailboxes   The array of mailboxes returned by
     *                            IMP_folder::flist().
     */
    function flist ($filter = false)
    {
        include_once './lib/folder.php';
        global $imp, $conf, $prefs;
        return IMP_folder::flist($imp['stream'], serverString(), $imp['folders'], $prefs->getValue('subscribe'), $filter, $conf['server']['hierarchies'], $conf['server']['show_dotfiles'], $conf['server']['namespace']);
    }
    
    /**
     * Wrapper around IMP::flist() which generates the body of a
     * &lt;select&gt; form input from the generated folder list. The
     * &lt;select&gt; and &lt;/select&gt; tags are NOT included in the output of
     * this function.
     *
     * @access public
     *
     * @param string $heading   (optional) An optional string to use as
     *                          the label for an empty-value option at
     *                          the top * of the list. Defaults to none.
     * @param bool $abbrev      (optional) If true, bbreviate long mailbox names
     *                          by replacing the middle of the name with
     *                          '...'. Defaults to true.
     * @param string $filter    (optional) A string specifying a mailbox to
     *                          ignore. Defaults to none.
     * @param string $selected  (optional) The mailbox to have selected by
     *                          default. Defaults to the first option in the
     *                          list.
     *
     * @return string $options  A string containg <option> elements for each
     *                          mailbox in the list.
     */
    function flistSelect ($heading = '', $abbrev = true, $filter = false, $selected = '')
    {
        $mailboxes = IMP::flist($filter);
        $options = '';
        
        if (strlen($heading) > 0)
            $options .= '<option value="">' . $heading . "</option>\n";
        
        foreach ($mailboxes as $mbox) {
            $sel = ($mbox['label'] == $selected) ? ' selected="selected"' : '';
            if ($abbrev)
                $options .= '<option value="' . $mbox['val'] . '"' . "$sel>" . $mbox['abbrev'] . "</option>\n";
            else
                $options .= '<option value="' . $mbox['val'] . '"' . "$sel>" . $mbox['label'] . "</option>\n";
        }
        
        return $options;
    }
    
    /**
     * Return an array of unsubscribed folders. This is a wrapper
     * around the IMP_folder::flistUnsubscribed() function which
     * reduces the number of arguments needed if we can assume that
     * IMP's full environment is present.
     *
     * @access public
     *
     * @param string $filter  (optional) A string specifying a mailbox to
     *                        ignore.
     *
     * @return array $mailboxes  The array of mailboxes returned by
     *                           IMP_folder::flistUnsubscribed().
     */
    function flistUnsubscribed ($filter = false)
    {
        include_once './lib/folder.php';
        global $imp, $conf;
        return IMP_folder::flistUnsubscribed($imp['stream'], serverString(), $imp['folders'], $filter, $conf['server']['hierarchies'], $conf['server']['show_dotfiles'], $conf['server']['namespace']);
    }
    
    /**
     * Wrapper around IMP::flistUnsubscribed() which generates the
     * body of a &lt;select&gt; form input from the generated unsubscribed
     * folder list. The &lt;select&gt; and &lt;/select&gt; tags are NOT included
     * in the output of this function.
     *
     * @access public
     *
     * @param string $heading   (optional) An optional string to use as
     *                          the label for an empty-value option at
     *                          the top * of the list. Defaults to none.
     * @param bool $abbrev      (optional) If true, bbreviate long mailbox names
     *                          by replacing the middle of the name with
     *                          '...'. Defaults to true.
     * @param string $filter    (optional) A string specifying a mailbox to
     *                          ignore. Defaults to none.
     * @param string $selected  (optional) The mailbox to have selected by
     *                          default. Defaults to the first option in the
     *                          list.
     *
     * @return string $options  A string containg <option> elements for each
     *                          mailbox in the list.
     */
    function flistUnsubscribedSelect ($heading = '', $abbrev = true, $filter = false, $selected = '')
    {
        $mailboxes = IMP::flistUnsubscribed($filter);
        $options = '';
        
        if (strlen($heading) > 0)
            $options .= '<option value="">' . $heading . "</option>\n";
        
        foreach ($mailboxes as $mbox) {
            $sel = ($mbox['label'] == $selected) ? ' selected="selected"' : '';
            if ($abbrev)
                $options .= '<option value="' . $mbox['val'] . '"' . $sel . '>' . $mbox['abbrev'] . "</option>\n";
            else
                $options .= '<option value="' . $mbox['val'] . '"' . $sel . '>' . $mbox['label'] . "</option>\n";
        }
        
        return $options;
    }
    
    /**
     * Returns the appropriate link to call the message composition screen.
     *
     * @access public
     *
     * @param array $args    (optional) Hash of arguments to pass to compose.php.
     * @param array $extra   (optional) Hash of extra, non-standard arguments to
     *                       pass to compose.php.
     *
     * @return string $link  The link to the message composition screen.
     */
    function composeLink($args = array(), $extra = array())
    {
        global $prefs, $registry;
        
        if ($prefs->getValue('compose_popup')) {
            /* This is NOT an elegant way to do it, but %27 needs to
               be escaped or else IE's jscript will interpret it as a
               single quote and refuse to work. */
            return str_replace('%27', '\%27', $registry->call('mail/compose', $args, $extra));
        } else {
            /* Merge the two argument arrays. */
            $args = array_merge($args, $extra);
            
            /* Convert the $args hash into proper URL parameters. */
            $params = '?';
            foreach ($args as $key => $val) {
                $key = urlencode($key);
                $val = urlencode($val);
                $params .= "$key=$val&";
            }
            
            /* Check for page or message number information to let the
               compose window return to the right place in the
               mailbox. */
            global $msgindex, $page;
            if (isset($msgindex)) {
                $params .= 'start=' . urlencode(($msgindex + 1)) . '&';
            }
            if (isset($page)) {
                $params .= 'page=' . urlencode($page) . '&';
            }
            
            return Horde::url('compose.php' . substr($params, 0, -1));
        }
    }
    
    /**
     * Returns an html table row summarizing a part of a MIME message.
     *
     * @access public
     * @param array $mime    The MIME object to summarize.
     * @return string $row   The html table row summary.
     */
    function partSummary($mime)
    {
        if (!($mime->size > 0)) return ''; // don't display zero-size attachments
        
        global $conf;
        
        switch ($mime->conf['action']) {
        case 'function':
            $func = $mime->conf['function'];
            $row = $func($mime);
            break;
            
        default:
            include_once '../lib/MIME.php';
            
            // icon column
            $row = '<tr valign="middle"><td>' .
                (($mime->conf['icon'] != '') ? 
                 '<img src="' . $conf['paths']['graphics'] . '/' . $mime->conf['icon'] . '" border="0" alt="' . $mime->TYPE . '/'. $mime->subtype . '" />' :
                 '&nbsp;') . '</td>';
            
            // number column
            $row .= '<td>' . $mime->imap_id . '</td>';
            
            // name/text part column
            $row .= '<td>';
            if ($mime->conf['view'])
                $row .= "<a href=\"javascript:view('" . VIEW_ATTACH . "', '" . $mime->imap_id . "', '" . SessionCache::putObject($mime) . "');\">" . htmlspecialchars(MIME::decode($mime->description)) . '</a>';
            else
                $row .= htmlspecialchars(MIME::decode($mime->description));
            $row .= '</td>';
            
            // type column
            $row .= '<td>' . $mime->TYPE . '/' . $mime->subtype . '</td>';
            
            // size column
            $row .= '<td>' . $mime->size . ' KB</td>';
            
            // download column
            if ($mime->conf['download']) {
                global $imp, $HTTP_GET_VARS; $mailbox = $imp['mailbox'];
                $row .= '<td><a href="' . Horde::url('view.php/' . urlencode($mime->name) .
                    '?mailbox=' . urlencode($mailbox)) . '&amp;index=' . $HTTP_GET_VARS['index'] .
                    "&amp;id=$mime->imap_id&amp;actionID=" . DOWNLOAD_ATTACH . '&amp;mime=' .
                    SessionCache::putObject($mime) . '"><img src="' .
                    $conf['paths']['graphics'] . '/download.gif" alt="' . _("Download") . '" border="0" /></a></td>';
            } else {
                $row .= '<td>&nbsp;</td>';
            }
            
            $row .= "</tr>\n";
            break;
        }
        
        return $row;
    }
    
}


class IMP_Util {
    
    function getAction()
    {
        global $HTTP_POST_VARS, $HTTP_GET_VARS;
        
        if (isset($HTTP_POST_VARS['actionID'])) {
            return $HTTP_POST_VARS['actionID'];
        } elseif (isset($HTTP_GET_VARS['actionID'])) {
            return $HTTP_GET_VARS['actionID'];
        } else {
            return NO_ACTION;
        }
    }
    
    function getPart($mime)
    {
        global $imp;
        return imap_fetchbody($imp['stream'], $mime->index, $mime->imap_id, FT_UID);
    }
    
    function getDecodedPart($mime)
    {
        global $imp;
        
        if ($mime->encoding == ENCBASE64) {
            return imap_base64(imap_fetchbody($imp['stream'], $mime->index, $mime->imap_id, FT_UID));
        } elseif ($mime->encoding == ENCQUOTEDPRINTABLE) {
            $raw = imap_fetchbody($imp['stream'], $mime->index, $mime->imap_id, FT_UID);
            $data = imap_qprint($raw);
            if (empty($data))
                $data = $raw;
            return $data;
        } else {
            return imap_fetchbody($imp['stream'], $mime->index, $mime->imap_id, FT_UID);
        }
    }
    
}


function getResult (&$result) {
    return (isset($result)) ? $result : '';
}

function get_barefrom() {
    global $conf, $imp, $prefs, $_barefrom;
  
    if ($prefs->isChangeable('from_addr') && empty($_barefrom)) {
        $_barefrom = $prefs->getValue('from_addr');
    }
  
    if (empty($_barefrom)) {
        if ($conf['hooks']['from_hook'] &&
            function_exists('imp_expand_fromaddress')) {
            $_barefrom = imp_expand_fromaddress($imp);
        } else {
            $_barefrom = $imp['user'];
        }
    }
  
    /* Allow proper use of user@server usernames. */
    if (!strstr($_barefrom, '@')) {
        $_barefrom .= '@' . $imp['maildomain'];
    }
  
    return $_barefrom;
}

function addSiteHeaders($headers)
{
    global $conf;
    
    // Tack on any site-specific headers.
    if ($conf['msg']['append_header'] && @is_readable('./config/header.txt')) {
        $lines = @file('./config/header.txt');
        foreach ($lines as $line) {
            $line = Text::expandEnvironment($line);
            if (!empty($line)) {
                list($key, $val) = explode(':', $line);
                $headers[$key] = trim($val);
            }
        }
    }
    
    return $headers;
}


function preambleString() {
    global $imp, $conf;
    return $imp['folders'] . $conf['server']['namespace'];
}

function serverString ($protocol = '') {
    global $imp;
    
    if ($protocol == '') {
        $protocol = $imp['protocol'];
    }
    return '{' . $imp['server'] . '/' . $protocol . ':' . $imp['port'] . '}';
}

function logoutURL ($uri) {
    global $conf, $imp;
    
    if (!empty($imp['user']))
        $uri .= '&imapuser=' . urlencode($imp['user']);
    elseif (!empty($GLOBALS['imapuser']))
        $uri .= '&imapuser=' . urlencode($GLOBALS['imapuser']);
    
    if ($conf['server']['server_list'] && !empty($GLOBALS['server']))
        $uri .= '&server=' . urlencode($GLOBALS['server']);
    elseif (!empty($imp['server']))
        $uri .= '&server=' . urlencode($imp['server']);
    
    if (!empty($imp['port']))
        $uri .= '&port=' . urlencode($imp['port']);
    if (!empty($imp['protocol']))
        $uri .= '&protocol=' . urlencode($imp['protocol']);
    if (!empty($imp['folders']))
        $uri .= '&folders=' . urlencode($imp['folders']);
    if (!empty($imp['language']))
        $uri .= '&language=' . urlencode($imp['language']);
    
    return $uri;
}

?>
