<?php
/*
 * $Horde: imp/login.php,v 2.85 2001/01/04 15:51:03 chuck Exp $
 *
 * Copyright 1999, 2000 Charles J. Hagenbuch <chuck@horde.org>
 * Copyright 1999, 2000 Jon Parise <jon@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

/**
 * Determines if the given mail server is the "preferred" mail server for
 * this web server.  This decision is based on the global $SERVER_NAME and
 * $HTTP_HOST environmental variables and the contents of the 'preferred'
 * field in the server's definition.  The 'preferred' field may take either
 * a single value or an array of multiple values.
 *
 * @param $server       A complete server entry from the $servers hash.
 *
 * @return  boolean     True if this entry is "preferred".
 */
function isPreferred($server)
{
    global $SERVER_NAME, $HTTP_HOST;

    if (is_array($server['preferred'])) {
        foreach ($server['preferred'] as $preferred) {
            if ($preferred == $SERVER_NAME || $preferred == $HTTP_HOST) {
                return true;
            }
        }
    } elseif ($server['preferred'] == $SERVER_NAME ||
              $server['preferred'] == $HTTP_HOST) {
        return true;
    }

    return false;
}

require_once './lib/base.php';
if ($conf['connections']['track']) {
    include_once '../lib/Connection.php';
}

/* Additional configuration (dependent on the above code libraries) */
require './config/servers.php';

if ($conf['log']['enabled']) {
    include_once 'Log.php';
    $log = &Log::singleton($conf['log']['type'], $conf['log']['name'],
						   $conf['log']['ident'], $conf['log']['conf']);
}

/* Map the various values for $reason to more descriptive status messages. */
$reasons = array('login'   => '',
                 'timeout' => _("Either you have logged in incorrectly or your login has expired. Please login again."),
                 'logout'  => sprintf(_("You have been logged out of %s. Thank you for using the system. If you wish to log in again, please use the form below."), $conf['sitename']),
                 'failed'  => _("Login failed for some reason. Most likely your username or password was entered incorrectly."));

/* Default to 'login' if no $reason is set. */
if (!isset($HTTP_GET_VARS['reason'])) $HTTP_GET_VARS['reason'] = 'login';

$actionID = IMP_util::getAction();

/* Initialize the session routines. */
session_name($horde['session_name']);
@session_start();

if (isset($HTTP_SESSION_VARS['imp'])) {
    $imp = &$HTTP_SESSION_VARS['imp'];
    if ($HTTP_GET_VARS['reason'] == 'logout') {
        $prefs = Prefs::factory($conf['prefs']['driver'], $imp['user'],
            $conf['prefs']['params']);
        $prefs->store();
        session_unregister('imp');
        if (empty($auth->auth['uname']) || $auth->auth['uname'] == 'nobody') {
            @session_destroy();
        }
    } else {
        /* If there is an existing session, redirect the user to the mailbox. */
        if (isset($actionID) && $actionID == IMP_LOGIN) {
            header('Location: ' . Horde::url('mailbox.php?actionID=' . IMP_LOGIN, true));
        } elseif (($actionID == LOGIN_COMPOSE) || (isset($HTTP_GET_VARS['action']) && ($HTTP_GET_VARS['action'] == 'compose'))) {
            header('Location: ' . Horde::url('mailbox.php?actionID=' . LOGIN_COMPOSE, true));
        } else {
            header('Location: ' . Horde::url('mailbox.php', true));
            exit;
        }
    }
}

/* Log the reason if logging is enabled. */
if ($conf['log']['enabled']) {
    $log->log($HTTP_SERVER_VARS['REMOTE_ADDR'] . ' ' . $HTTP_GET_VARS['reason'], LOG_INFO);
}

/* Redirect the user on logout if redirection is enabled. */
if (($HTTP_GET_VARS['reason'] == 'logout') && $conf['user']['redirect_on_logout']) {
    header('Location: ' . $conf['user']['redirect_on_logout']);
    exit;
}

/* Redirect the user if an alternate login page has been specified. */
if ((in_array($HTTP_GET_VARS['reason'], array('login', 'failed', 'timeout'))) && $conf['user']['alternate_login']) {
    header('Location: ' . $conf['user']['alternate_login']);
    exit;
}

/* Set the actionID of the login form */
if ((isset($HTTP_GET_VARS['action']) && $HTTP_GET_VARS['action'] == 'compose') ||
    ($actionID == LOGIN_COMPOSE)) {
    $actionID = LOGIN_COMPOSE;
} else {
    $actionID = IMP_LOGIN;
}

/* Get the default preferences values. */
$prefs = Prefs::factory('none');
$prefs->setDefaults('./config/prefs.php');

/* Grab some default values from the first entry in config/servers.php. */
reset($servers);
$server_key = key($servers);
$server_value = $servers[$server_key]['server'];
$protocol_value = $servers[$server_key]['protocol'];
$port_value = $servers[$server_key]['port'];
$folders_value = $servers[$server_key]['folders'];

/*
 * If the server list is used but should be hidden (for virtual hosting),
 * make sure we output a server name to use.
 */
if ($conf['server']['server_list'] && $conf['server']['server_list_hidden']) {
    /*
     * Iterate through the servers in an attempt to locate a "preferred"
     * server for this web server.  If none if found, we default to the
     * first entry in the hash (indicated by the $server_key value set
     * above).
     */
    reset($servers);
    while (list($key, $this_server) = each($servers)) {
        if (isPreferred($this_server)) {
            $server_key = $key;
            break;
        }
    }
}

/* Build the <select> widget for the servers list. */
if ($conf['server']['server_list'] && !$conf['server']['server_list_hidden']) {
    $servers_list = '';

    reset($servers);
    while (list($key, $this_server) = each($servers)) {
        $sel = (isPreferred($this_server)) ? ' selected="selected"' : '';
        $servers_list .= "<option value=\"$key\"$sel>";
        $servers_list .= $this_server['name'] . "</option>\n";
    }
}

/* Build the <select> widget containing the available languages. */
if ($prefs->isChangeable('language')) {
    $langs = '<select name="new_lang" onchange="selectLang()">';
    while (list($key, $val) = each($nls['languages'])) {
        $sel = ($key == $language) ? ' selected="selected"' : '';
        $langs .= "<option value=\"$key\"$sel>$val</option>\n";
    }
    $langs .= '</select>';
}

$title = _('Welcome');
$js_onLoad = 'setFocus()';
include $conf['paths']['templates'] . '/doctype.inc';
include $conf['paths']['templates'] . '/common-header.inc';
include $conf['paths']['templates'] . '/login/login.inc';
if (@is_readable('./config/MOTD.html')) {
    include './config/MOTD.html';
}
include $conf['paths']['templates'] . '/common-footer.inc';

?>
