<?php
/*
 * $Horde: imp/mailbox.php,v 2.234 2001/01/10 19:24:21 chuck Exp $
 *
 * Copyright 1999, 2000 Charles J. Hagenbuch <chuck@horde.org>
 * Copyright 1999, 2000 Jon Parise <jon@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

/*
 * Check the user's address and the list of addresses for which To: should
 * be displayed.
 */
function display_toaddress($user, $server, $from) {
    return is_int(strpos($from, get_barefrom($user, $server)));
}

function runSearch (&$sorted) {
    global $imp, $folders;
    
    $sorted = array();
    if (empty($imp['searchquery']) || empty($imp['searchfolders'])) {
        return array();
    }
    
    for ($i = 0; $i < count($imp['searchfolders']); $i++) {
        if ($imp['protocol'] == 'pop3') {
            $stream = $imp['stream'];
        } else {
            $stream = @imap_open(serverString() . $imp['searchfolders'][$i],
                                 $imp['user'], Secret::read(IMP::getKey(), $imp['pass']), OP_READONLY);
        }
        
        if ($stream) {
            $results = imap_search($stream, $imp['searchquery'], SE_UID);
            if (is_array($results)) {
                for ($j = 0; $j < count($results); $j++) {
                    $sorted[] = $results[$j];
                    $folders[] = $imp['searchfolders'][$i];
                }
            }
            if ($imp['protocol'] != 'pop3') {
                imap_close($stream);
            }
        }
    }
    
    if (isset($folders)) {
        $imp['messagefolders'] = implode(':', $folders);
        return implode(':', $sorted);
    } else {
        return false;
    }
}

function stylize($string, $styles)
{
    if (!is_array($styles)) return $string;

    foreach ($styles as $style) {
        if (!empty($style)) {
            $string = '<' . $style . '>' . $string . '</' . $style . '>';
        }
    }
    
    return $string;
}

$set = false;
if (isset($new_lang)) {
    $language = $new_lang;
    $set = true;
}

require_once './lib/base.php';
require_once './lib/message.php';
require_once '../lib/MIME.php';
require_once '../lib/Registry.php';
require './config/mime.php';

$registry = new Registry();

if ($conf['log']['enabled']) {
    include_once 'Log.php';
    $log = &Log::singleton($conf['log']['type'], $conf['log']['name'], $conf['log']['ident'], $conf['log']['conf']);
}

session_name($horde['session_name']);
@session_start();

/* Initialize $actionID to a resonable value if it hasn't already been set. */
$actionID = IMP_util::getAction();

if (isset($HTTP_SESSION_VARS['imp'])) {
    IMP::setupSession();
} else {
    /* Do this if logging in ... */
    if (($actionID == IMP_LOGIN) || ($actionID == LOGIN_COMPOSE)) {
        IMP::createSession();
    }
}

/* If we authenticate ... */
if (IMP::authenticate()) {
    if (($actionID == IMP_LOGIN) || ($actionID == LOGIN_COMPOSE)) {
        if (isset($new_lang)) {
            $language = $new_lang;
        }
        
        if ($conf['log']['enabled']) {
            $entry = sprintf('%s login success for %s {%s:%s}',
                             $HTTP_SERVER_VARS['REMOTE_ADDR'], $imp['user'], $imp['server'], $imp['port']);
            $log->log($entry, LOG_NOTICE);
        }
        
        if ($actionID == LOGIN_COMPOSE) {
            if ($prefs->getValue('compose_popup')) {
                $open_compose_window = true;
            } else {
                header('Location: ' . Horde::url('compose.php'));
            }
        }
    }
} else {
    header('Location: ' . Horde::url(logoutURL('login.php?reason=failed'), true));
}


/* Massage data coming in via GET/POST to be where we expect it. */
if (isset($HTTP_POST_VARS['indices'])) {
    $indices = $HTTP_POST_VARS['indices'];
    if (!is_array($indices)) $indices = array($indices);
} elseif (isset($HTTP_GET_VARS['indices'])) {
    $indices = $HTTP_GET_VARS['indices'];
    if (!is_array($indices)) $indices = array($indices);
}
if (isset($HTTP_POST_VARS['targetMbox'])) {
    $targetMbox = $HTTP_POST_VARS['targetMbox'];
} elseif (isset($HTTP_GET_VARS['targetMbox'])) {
    $targetMbox = $HTTP_GET_VARS['targetMbox'];
}

/* If this is a login, and the user wants to run filters on login,
   make sure they get run. */
if ($actionID == IMP_LOGIN && $prefs->getValue('filter_on_login')) {
    $actionID = FILTER;
}

/* Run through the action handlers */
switch ($actionID) {
 case NO_ACTION:
     break;
     
 case DELETE_MESSAGES:
     if (!empty($indices)) {
         IMP_message::delete($indices);
     }
     break;
        
 case UNDELETE_MESSAGES:
     if (!empty($indices)) {
         IMP_message::undelete($indices);
     }
     break;
        
 case MOVE_MESSAGES:
     if (!empty($indices) && !empty($targetMbox)) {
         IMP_message::copy($targetMbox, $indices, MOVE_MESSAGES);
     }
     break;
        
 case COPY_MESSAGES:
     if (!empty($indices) && !empty($targetMbox)) {
         IMP_message::copy($targetMbox, $indices, COPY_MESSAGES);
     }
     break;
        
 case FLAG_MESSAGES:
     if (!empty($indices) && !empty($HTTP_POST_VARS['flag'])) {
         if ($HTTP_POST_VARS['flag'][0] == '0') {
             $HTTP_POST_VARS['flag'] = '\\' . substr($HTTP_POST_VARS['flag'], 1);
             $set = false;
         } else {
             $HTTP_POST_VARS['flag'] = '\\' . $HTTP_POST_VARS['flag'];
             $set = true;
         }
         IMP_message::flag($HTTP_POST_VARS['flag'], $indices, $set);
     }
     break;
        
 case HIDE_DELETED:
     $prefs->setValue('delhide', !$prefs->getValue('delhide'));
     break;
        
 case EXPUNGE_MAILBOX:
     if ($imp['mailbox']) {
         if (!(imap_expunge($imp['stream']))) {
             Horde::raiseMessage(_("There was a problem expunging the mailbox. This is what the server said") .
                                 ': <i>' . imap_last_error() . '</i>', HORDE_ERROR);
         }
     }
     break;
     
 case SEARCH:
     /* Create the search query with the given data. */
     $query = '';
     switch ($HTTP_GET_VARS['search_seen']) {
     case 0:
         $query .= ' SEEN';
         break;
     case 1:
         $query .= ' UNSEEN';
         break;
     default:
         break;
     }
     
     switch ($HTTP_GET_VARS['search_answered']) {
     case 0:
         $query .= ' ANSWERED';
         break;
     case 1:
         $query .= ' UNANSWERED';
         break;
     default:
         break;
     }
     
     if (!empty($HTTP_GET_VARS['search_subject']))
         $query .= ' SUBJECT "' . $search_subject . '"';
        
     if (!empty($HTTP_GET_VARS['search_from']))
         $query .= ' FROM "' . $search_from . '"';
        
     if (!empty($HTTP_GET_VARS['search_to']))
         $query .= ' TO "' . $HTTP_GET_VARS['search_to'] . '"';
        
     if (!empty($HTTP_GET_VARS['search_body']))
         $query .= ' BODY "' . $HTTP_GET_VARS['search_body'] . '"';
        
     $imp['searchquery'] = trim($query);
     $imp['searchfolders'] = $HTTP_GET_VARS['search_folders'];
     break;

 case FILTER:
     $filters = @unserialize($prefs->getValue('filters'));
     if (is_array($filters) && count($filters) && $imp['protocol'] != 'pop3') {
         include_once './lib/folder.php';
         $baseQuery = 'UNDELETED ';
         for ($i = 0; $i < count($filters); $i++) {
             $query = $baseQuery . strtoupper($filters[$i]['field']) . ' "' . $filters[$i]['text'] . '"';
             $indices = imap_search($imp['stream'], $query, SE_UID);
             if (isset($indices) && is_array($indices)) {
                 if ($filters[$i]['action'] == 'delete') {
                     IMP_message::delete($indices);
                 }
                 else {
                     if (IMP_folder::exists($imp['stream'], $filters[$i]['folder'])) {
                         IMP_message::copy($filters[$i]['folder'], $indices, MOVE_MESSAGES);
                     } else {
                         Horde::raiseMessage(_(sprintf("%s %d: %s: ", "problem with filter rule", $i + 1, "folder does not exist")) . IMP_folder::displayName($filters[$i]['folder']), HORDE_WARNING);
                     }
                 }
             }
         }
     }
     break;
}


if ($conf['user']['allow_folders']) {
    $options = IMP::flistSelect(_("Messages to"), true, array($imp['mailbox']));
}

if ($imp['mailbox'] == '**search') {
    $imp['msgl'] = runSearch($sorted);
    $imp['msgcount'] = count($sorted);
} else {
    if ($prefs->getValue('delhide')) {
        $msgs = imap_search($imp['stream'], 'UNDELETED');
        $imp['msgcount'] = ($msgs !== false) ? count($foo) : 0;
    } else {
        $check = imap_check($imp['stream']);
        $imp['msgcount'] = (is_object($check) && isset($check->Nmsgs)) ? $check->Nmsgs : 0;
    }
}

/* Make sure we have a reasonable value for 'max_msgs'. */
if ($prefs->getValue('max_msgs') <= 0) {
    $prefs->setValue('max_msgs', 20);
}

/* Paging logic. This should be cleaned up if at all possible. */
if ($imp['msgcount'] > $prefs->getValue('max_msgs')) {
    
    $page_count = ceil($imp['msgcount'] / $prefs->getValue('max_msgs'));
    
    /* Determine which page to display */
    if (isset($HTTP_GET_VARS['page']) && !strcspn($HTTP_GET_VARS['page'], '0123456789')) {
        /* This will be sanity checked later. */
        $page = $HTTP_GET_VARS['page'];
        
    } elseif (isset($HTTP_POST_VARS['page']) && !strcspn($HTTP_POST_VARS['page'], '0123456789')) {
	/* This will be sanity checked later. */
	$page = $HTTP_POST_VARS['page'];
	
    } elseif (!empty($HTTP_GET_VARS['start'])) {
        /* messages set this when returning to a mailbox */
        $page = ceil($HTTP_GET_VARS['start'] / $prefs->getValue('max_msgs'));
        
    } elseif ($imp['mailbox'] != '**search') {
        /* try and use imap_search to find the first unread message */
        $new = imap_search($imp['stream'], 'UNSEEN');
        if ($new) {
            if ($prefs->getValue('sortdir')) {
                $page = $page_count - ceil((max($new)) /
                                           $prefs->getValue('max_msgs')) + 1;
            } else {
                $page = ceil(min($new) / $prefs->getValue('max_msgs'));
            }
        } else {
            $page = $prefs->getValue('sortdir') ? 1 : $page_count;
        }
    } else {
        $page = $prefs->getValue('sortdir') ? 1 : $page_count;
    }
    
    /* make sure we're not past the end or before the beginning. */
    if ($page > $page_count) {
        $page = $page_count;
    } elseif ($page < 1) {
        $page = 1;
    }
    
    $begin = (($page - 1) * $prefs->getValue('max_msgs')) + 1;
    $end = $begin + $prefs->getValue('max_msgs') - 1;
    if ($end > $imp['msgcount']) {
        $end = $imp['msgcount'];
    }
    
    if ($page == 1) {
        if ($browser->hasFeature('images')) {
            $pages_first = Horde::img('first-grey.gif', 'alt=""');
            $pages_prev = Horde::img('prev-grey.gif', 'alt=""');
        } else {
            $pages_first = '';
            $pages_prev  = '';
        }
    } else {
        $prev = $page - 1;
        $target_first_url = Horde::url('mailbox.php?page=1');
        $target_prev_url  = Horde::url("mailbox.php?page=$prev");
        if ($browser->hasFeature('images')) {
            $pages_first = "<a href=\"$target_first_url\">";
            $pages_first .= Horde::img('first.gif', 'alt="' . _("First Page") . '"');
            $pages_first .= '</a>';
            $pages_prev = "<a href=\"$target_prev_url\">";
            $pages_prev .= Horde::img('prev.gif', 'alt="' . _("Previous Page") . '"');
            $pages_prev .= '</a>';
        } else {
            $pages_first = "<a href=\"$target_first_url\">&lt;&lt; [ " .
                _("First Page") . ' ]';
            $pages_prev  = "<a href=\"$target_prev_url\">&lt; [ " .
                _("Previous Page") . ' ]</a>';
        }
    }
    
    if ($page == $page_count) {
        if ($browser->hasFeature('images')) {
            $pages_last = Horde::img('last-grey.gif', 'alt=""');
            $pages_next = Horde::img('next-grey.gif', 'alt=""');
        } else {
            $pages_last = '';
            $pages_next = '';
        }
    } else {
        $next = $page + 1;
        $target_last_url = Horde::url("mailbox.php?page=$page_count");
        $target_next_url = Horde::url("mailbox.php?page=$next");
        
        if ($browser->hasFeature('images')) {
            $pages_last = "<a href=\"$target_last_url\">";
            $pages_last .= Horde::img('last.gif', 'alt="' . _("Last Page") . '"');
            $pages_last .= '</a>';
            $pages_next = "<a href=\"$target_next_url\">";
            $pages_next .= Horde::img('next.gif', 'alt="' . _("Next Page") . '"');
            $pages_next .= '</a>';
        } else {
            $pages_last = "<a href=\"$target_last_url\">[ " .
                _("Last Page") . ' ] &gt;&gt;</a>';
            $pages_next = "<a href=\"$target_next_url\">[ " .
                _("Next Page") . ' ] &gt;</a>';
        }
    }
} else {
    $begin = 1;
    $end = $imp['msgcount'];
    $page = 1;
}

if ($imp['mailbox'] == '**search') {
    $imp['offset'] = 0;
    $beginIndex = $begin - 1;
} else {
    if ($imp['msgcount'] != 0) {
        if ($prefs->getValue('delhide')) {
            $sorted = imap_sort($imp['stream'], $prefs->getValue('sortby'),
                                $prefs->getValue('sortdir'), SE_UID, 'UNDELETED');
        } else {
            $sorted = imap_sort($imp['stream'], $prefs->getValue('sortby'),
                                $prefs->getValue('sortdir'), SE_UID);
        }
    } else {
        $sorted = array();
    }
    
    $imp['msgl'] = implode(':', IMP_message::range($sorted, $begin, $offset, $beginIndex, $num));
    $imp['offset'] = $offset;
}

if ($prefs->getValue('delhide')) {
    $deleted_prompt = _("Show Deleted");
} else {
    $deleted_prompt = _("Hide Deleted");
}

if ($end != 0) {
    $msg_count = sprintf(_("%d to %d of %d Messages"), $begin, $end, $imp['msgcount']);
} else {
    $msg_count = sprintf(_("%d of %d Messages"), $end, $imp['msgcount']);
}

/* If user wants the mailbox to be refreshed, set time here */
$refresh_time = $prefs->getValue('refresh_time');
$refresh_url = Horde::url("mailbox.php?page=$page&uniq=".uniqid(rand()));

$title = $imp['label'];
if ($conf['compress_pages']) {
    // HTTP_Cache::start();
}

require $conf['paths']['templates'] . '/doctype.inc';
require $conf['paths']['templates'] . '/common-header.inc';
require './menu.php';
require $conf['paths']['templates'] . '/mailbox/header.inc';
require $conf['paths']['templates'] . '/mailbox/navbar.inc';
require $conf['paths']['templates'] . '/mailbox/actions.inc';
if ($imp['mailbox'] != '**search') {
    include $conf['paths']['templates'] . '/mailbox/message_headers.inc';
}

// now step through each message
$delnum = 0;
for ($i = $begin, $j = $beginIndex; $i <= $end; $i++, $j++) {
    
    /* Make sure that the index is actually in the slice of messages
       we're looking at. If we're hiding deleted messages, for
       example, there may be gaps here. */
    if (isset($sorted[$i - 1])) {
        $msgnum = $sorted[$i - 1];
        
        if ($imp['mailbox'] == '**search') {
            if (($j == $beginIndex) || ($folders[$j-1] != $folders[$j])) {
                if ($imp['protocol'] != 'pop3') {
                    imap_reopen($imp['stream'], serverString() . $folders[$j],
                                OP_READONLY);
                }
                
                include $conf['paths']['templates'] . '/mailbox/searchfolder.inc';
                include $conf['paths']['templates'] . '/mailbox/message_headers.inc';
            }
        }
        $h = imap_header($imp['stream'], imap_msgno($imp['stream'], $msgnum));
        
        /* Display the message ... */
        
        /* initialize the header fields */
        $dat = '';
        $frm = '';
        $fulfrom = '';
        $to = '';
        $sub = '';
        $msg_size = '?';
        
        /* Now pull the IMAP header values into them, decoding them at
           the same time. */
        
        /* formats the header date string nicely */
        if (isset($h->date)) {
            $udate = strtotime($h->date);
            if ((date('Y') != date('Y', $udate)) ||
                (date('M') != date('M', $udate)) ||
                (date('d') != date('d', $udate))) {
                // not today, use the date
                $dat = strftime($conf['mailbox']['date_format'], $udate);
            } else {
                // else it's today, use the time
                $dat = strftime($conf['mailbox']['time_format'], $udate);
            }
        } else {
            $dat = '';
        }
        
        if (isset($h->from[0])) {
            $fullfrom = '';

            if (isset($h->from[0]->personal)) {
                $fullfrom = $frm = MIME::decode($h->from[0]->personal);
            }

            if (isset($h->from[0]->mailbox) && isset($h->from[0]->host)) {
                if (empty($frm)) $frm = $h->from[0]->mailbox . '@' . $h->from[0]->host;
                $fullfrom .= $h->from[0]->mailbox . '@' . $h->from[0]->host;
            }
        }
        
        if (display_toaddress($imp['user'], $imp['server'], $fullfrom)) {
            if (isset($h->to[0])) {
                if (isset($h->to[0]->personal)) {
                    $to = MIME::decode($h->to[0]->personal);
                }
                if (empty($to) && isset($h->to[0]->mailbox) && isset($h->to[0]->host)) {
                    $to = $h->to[0]->mailbox . '@' . $h->to[0]->host;
                }
            } else {
                $to = _("Undisclosed Recipients");
            }
            $frm = _("To") . ': ' . $to;
        }
        
        if (isset($h->subject)) {
            $sub = MIME::decode($h->subject);
        }
        if (isset($h->Size)) {
            $msg_size = ($h->Size > 1024)
                ? sprintf('%.0fkb', $h->Size / 1024)
                : $h->Size;
        }
        if (strlen($frm) > $conf['mailbox']['max_from_chars']) {
            $frm = substr($frm, 0, $conf['mailbox']['max_from_chars']) . '...';
        }
        if (strlen($sub) > $conf['mailbox']['max_subj_chars']) {
            $sub = substr($sub, 0, $conf['mailbox']['max_subj_chars']) . '...';
        }
        
        /* make sure that they get put into an HTML readable format */
        $frm = htmlspecialchars($frm);
        $sub = htmlspecialchars($sub);
        
        /* set them to useful things for IMP if they ended up being empty */
        if ($dat == '') { $dat = '&nbsp;'; }
        if ($frm == '') { $frm = '&nbsp;'; }
        if ($sub == '') { $sub = '[' . _("No subject") . ']'; }
        
        /* filter the subject text, if requested */
        if ($prefs->getValue('filtering')) {
            $sub = Text::filter($sub, $conf['msg']['filtering']['words'],
                                $conf['msg']['filtering']['replacement']);
        }
        
        $attachment = false;
        $structure = @imap_fetchstructure($imp['stream'], $msgnum, FT_UID);
        if (is_object($structure)) {
            if (!empty($structure->subtype)) {
                if (strtolower($structure->subtype) == 'signed' ||
                    strtolower($structure->subtype) == 'encrypted') {
                    $attachment = 'signed';
                } elseif (strtolower($structure->subtype) == 'alternative') {
                    $attachment = 'alternative';
                } elseif (isset($structure->parts) &&
                          is_array($structure->parts) &&
                          count($structure->parts) > 0) {
                    $attachment = 'attachment';
                } elseif ((!empty($structure->type) &&
                           $structure->type != TYPETEXT) ||
                          !empty($structure->subtype) &&
                          strtolower($structure->subtype) != 'plain') {
                    $attachment = 'attachment';
                }
            }
        }
        
        if ($imp['mailbox'] == '**search') {
            $target = Horde::url('message.php?mailbox=**search&msgmailbox=' .
                                 urlencode($folders[$i-1]) . '&index=' . urlencode($msgnum) .
                                 '&array_index=' . $j);
        } else {
            $target = Horde::url('message.php?index=' . urlencode($msgnum) .
                                 '&array_index=' . $j);
        }
        
        /* get all the flag information */
        $flagbits = 0;
        $style = array();
        $status = '';
        $bg = 'text';
        if ($imp['protocol'] != 'pop3') {
            if (!empty($h->toaddress) && strstr(strtolower($h->toaddress), strtolower(get_barefrom()))) {
                $status .= $conf['mailbox']['personal_flag'];
                $flagbits |= IMP_PERSONAL;
            }
            if ($h->Unseen == 'U' || $h->Recent == 'N') {
                $flagbits |= IMP_UNSEEN;
                $status .= $conf['mailbox']['unseen_flag'];
                $style[] = $conf['mailbox']['unseen_style'];
                $bg = 'unseen';
            }
            if ($h->Answered == 'A') {
                $flagbits |= IMP_ANSWERED;
                $status .= $conf['mailbox']['answered_flag'];
                $style[] = $conf['mailbox']['answered_style'];
                $bg = 'answered';
            }
            if (isset($h->Draft) && $h->Draft == 'X') {
                $flagbits |= IMP_DRAFT;
                $status .= $conf['mailbox']['draft_flag'];
                $style[] = $conf['mailbox']['draft_style'];
                $target = IMP::composeLink(array(), array('actionID' => DRAFT, 'mailbox' => $imp['mailbox'], 'index' => $msgnum, 'bodypart' => 1));
            }
            if ($h->Flagged == 'F') {
                $flagbits |= IMP_FLAGGED;
                $status .= $conf['mailbox']['important_flag'];
                $style[] = $conf['mailbox']['important_style'];
                $bg = 'important';
            }
            if ($h->Deleted == 'D') {
                $flagbits |= IMP_DELETED;
                $status .= $conf['mailbox']['deleted_flag'];
                $style[] = $conf['mailbox']['deleted_style'];
                $bg = 'deleted';
            }
            $flags[] = $flagbits;
        } else {
            $flags[] = 0;
        }
        
        // apply styles
        $dat = stylize($dat, $style);
        $frm = stylize($frm, $style);
        $sub = stylize($sub, $style);
        
        // decide what to do in the from column (after we've possibly
        // applied styles to $from)
        if ($conf['mailbox']['from_link'] == 'message') {
            if ($imp['mailbox'] == '**search') {
                $from_link = '<a class="message" href="';
                $from_link .= Horde::url('message.php?mailbox=**search&msgmailbox=' .
                                 urlencode($folders[$i-1]) . '&index=' . urlencode($msgnum) .
                                 '&array_index=' . $j);
                $from_link .= '">' . $frm . '</a>';
            } else {
                $from_link = '<a class="message" href="' . Horde::url("message.php?index=$msgnum&array_index=$j") . '">' . $frm . '</a>';
            }
        } elseif ($conf['mailbox']['from_link'] == 'compose') {
            $compose_url = IMP::composeLink(array(), array('actionID' => MAILTO, 'mailbox' => $imp['mailbox'], 'index' => $msgnum));
            $from_link = "<a class=\"message\" href=\"$compose_url\">$frm</a>";
        } else {
            $from_link = $frm;
        }
        
        include $conf['paths']['templates'] . '/mailbox/message_summaries.inc';
    }
} // end message display loop

if ($delnum > 0) {
    include $conf['paths']['templates'] . '/mailbox/deleted_summary.inc';
}

if ($end == 0) {
    if ($imp['mailbox'] == '**search') {
        include $conf['paths']['templates'] . '/mailbox/message_headers.inc';
    }
    include $conf['paths']['templates'] . '/mailbox/empty_mailbox.inc';
}

require $conf['paths']['templates'] . '/mailbox/message_footers.inc';
if ($conf['mailbox']['show_legend']) {
    include $conf['paths']['templates'] . '/mailbox/legend.inc';
}
require $conf['paths']['templates'] . '/mailbox/actions.inc';
require $conf['paths']['templates'] . '/mailbox/navbar.inc';
require $conf['paths']['templates'] . '/mailbox/footer.inc';
if ($browser->hasFeature('javascript')) {
    include $conf['paths']['templates'] . '/mailbox/javascript.inc';
}

if ($conf['user']['online_help'] && $browser->hasFeature('javascript')) {
    Help::javascript();
}

$registry->shutdown();
if (!empty($open_compose_window)) {
    $registry->pinvoke('mail/compose', array());
}

require $conf['paths']['templates'] . '/common-footer.inc';

$prefs->store();

if ($conf['compress_pages']) {
    // HTTP_Cache::output();
}

?>
