<?php
/*
 * $Horde: imp/menu.php,v 2.66 2001/01/02 20:58:08 max Exp $
 *
 * Copyright 1999, 2000 Charles J. Hagenbuch <chuck@horde.org>
 * Copyright 1999, 2000 Jon Parise <jon@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

/**
 * Generates the HTML for an item on the menu bar.
 *
 * @param $url      String containing the value for the hyperlink.
 * @param $text     String containing the label for this menu item.
 * @param $icon     (optional) String containing the filename of the image icon
 *                  to display for this menu item.
 *
 * @return  string  String containing the HTML to display this menu item.
 */
function menuItem($url, $text, $icon = '')
{
    global $conf;

    $html = '<td align="center" valign="bottom" nowrap="nowrap">';
    $html .= '<a class="menuitem" href="' . $url . '" ';
    $html .= "onmouseover=\"status='$text'; return true;\" ";
    $html .= 'onmouseout="status=\'\';">';
    
    if (!empty($icon) && $conf['layout']['menu_images']) {
        $html .= Horde::img($icon, "alt=\"$text\"") . '<br />';
    }
    
    $html .= $text . "</a>&nbsp;</td>\n";
    
    return $html;
}

/**
 * Prints the result of the menuItem function.
 *
 * @param $url      String containing the value for the hyperlink.
 * @param $text     String containing the label for this menu item.
 * @param $icon     (optional) String containing the filename of the image icon
 *                  to display for this menu item.
 */
function pmenuItem($url, $text, $icon = '')
{
    echo menuItem($url, $text, $icon);
}

/**
 * Parses a line of text in the form:
 *
 *      url<label>icon
 *
 * ... and passes the components to the menuItem function.
 *
 * @param $text     String containing a line of text representing a menu item.
 *
 * @return  string  The resulting HTML to display the menu item.
 */
function configMenuItem($text)
{
    global $conf;

    /* Skip any comment lines. */
    if (preg_match("|([^#]*)#.*|", $text, $regs)) {
        $text = $regs[1];

        /* Salvage anything before the comment. */
        if (strlen($text) > 0) {
            $text = $text . "\n";
        }
    }

    if (stristr($text, '<separator>')) {
        $text = '<td>&nbsp;</td>';
    } elseif (preg_match("|(.*)<(.*)>(.*)\n|", $text, $regs)) {
        $text = menuItem($regs[1], $regs[2], $regs[3]);
    }

    return $text;
}

/* Get the folder listing for the <select> widget. */
if ($conf['user']['allow_folders']) {
    $menu_folders = IMP::flistSelect('', true, false, $imp['label']);
}

/* Check for additional site-specific menu items (in config/menu.txt). */
$additional_items = '';
$items = @file('./config/menu.txt');
if (is_array($items)) {
    foreach ($items as $item) {
        $additional_items .= configMenuItem($item);
    }
}

include $conf['paths']['templates'] . '/menu/menu.inc';
require './status.php';

?>
