<?php
/*
 * $Horde: imp/message.php,v 2.188 2001/01/10 19:24:22 chuck Exp $
 *
 * Copyright 1999, 2000 Charles J. Hagenbuch <chuck@horde.org>
 * Copyright 1999, 2000 Jon Parise <jon@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

require_once './lib/base.php';
require_once './lib/mimetypes.php';
require_once '../lib/MIME.php';
require_once '../lib/MIME/Structure.php';
require_once './lib/message.php';
require_once '../lib/Registry.php';
require './config/mime.php';

$registry = new Registry();

/**
 * Builds a string containing a list of addresses.  Each address is a link
 * to the compose function.
 *
 * @param $array            Array of address objects.
 *
 * @return  string          String containing the formatted address list.
 */
function buildAddressLinks($array)
{
    $ret = '';
    while (list(,$ob) = each($array)) {
        
        /* Ensure we're working with initialized values. */
        $ob->personal = isset($ob->personal) ? MIME::decode($ob->personal) : '';
        if (!isset($ob->mailbox)) $ob->mailbox = '';
        if (!isset($ob->host)) $ob->host = '';
        
        /* Get full address ($addr) and the trimmed, bare address ($inner). */
        $addr = IMP::addrObject2String($ob, 'undisclosed-recipients@');
        $inner = Mime::trimEmailAddress(imap_rfc822_write_address($ob->mailbox,
                    $ob->host, ''));
        
        /* Separate multiple addresses with a comma. */
        if (!empty($ret)) $ret .= ', ';
        
        if (!empty($addr) && !empty($inner)) {
            $ret .= '<a href="' . IMP::composeLink(array('to' => $addr)) . '" ';
            $ret .= 'onmouseover="status=\'' . _("Compose Message") . ' (';
            $ret .= htmlentities($inner) . ')\'; return true;"';
            $ret .= ' onmouseout="status=\'\';">' . htmlspecialchars($addr);
            $ret .= '</a>';
        }
    }
    
    /*
     * If we're left with an empty address list ($ret), inform the user that
     * the recipient list is purposely "undisclosed".
     */
    if (empty($ret)) {
        $ret = _('Undisclosed Recipients');
    }
    
    return $ret;
}

function buildMessage($attachments, $msg)
{
    global $imp, $conf, $inline_parts, $atc;
    $j = 0;
    
    foreach ($attachments as $ref => $mime) {
        if (!$mime->header) {
            if ($mime->type == TYPETEXT && 
                ($mime->subtype == 'plain' || $mime->subtype == 'enriched')
                && $mime->bytes > 0
                && ($mime->encoding == ENC7BIT || $mime->encoding == ENC8BIT || $mime->encoding == ENCQUOTEDPRINTABLE || $mime->encoding == ENCBASE64)
                && ($inline_parts == 0 || $conf['msg']['inline_text'])) {
                
                /* Safe to display, it is text */
                $tmsg = mime_view_text($mime);
                
                if ($inline_parts > 0) {
                    $msg .= '</td></tr><tr><td><font size="1"><br /></font><table border="0">' . IMP::partSummary($mime) . '</table></td></tr><tr><td class="text">' . $tmsg;
                } else {
                    $msg = $tmsg;
                }
                $inline_parts++;
                
            /*
             * It may not be text directly, but if its configured to be
             * displayed inline, inline it.
             */
            } elseif (!empty($mime->conf['view']) && !empty($mime->conf['inline']) &&
                      ($mime->encoding == ENC7BIT || $mime->encoding == ENC8BIT || $mime->encoding == ENCQUOTEDPRINTABLE || $mime->encoding == ENCBASE64)
                      && isset($mime->conf['view_function'])
                      && $mime->disposition == 'inline') {
                
                if ($inline_parts > 0) {
                    /* If this MIME-type is configured to override other things for display, do it */
                    if (isset($mime->conf['override_text']) && $mime->conf['override_text']) {
                        $msg = '';
                    } else {
                        $msg .= '</td></tr><tr><td><font size="1"><br /></font><table border="0">' . IMP::partSummary($mime) . '</table></td></tr><tr><td class="text">';
                    }
                    $func = $mime->conf['view_function'];
                    $msg .= $func($mime);
                } else {
                    $func = $mime->conf['view_function'];
                    $msg .= $func($mime);
                }
                $inline_parts++;
            } else {
                $atc .= IMP::partSummary($mime);
            }
        }
    }
    return $msg;
}

session_name($horde['session_name']);
@session_start();

if ((!isset($HTTP_SESSION_VARS['imp'])) || (!is_array($HTTP_SESSION_VARS['imp']))) {
    header('Location: ' . Horde::url('login.php?reason=logout', true));
    exit;
}
IMP::setupSession();

$sorted = explode(':', $imp['msgl']);
$actionID = IMP_util::getAction();

IMP::authenticate();
if ($imp['mailbox'] != '**search') {
    if ($HTTP_GET_VARS['array_index'] == count($sorted) - 1 ||
        $HTTP_GET_VARS['array_index'] == 0 ||
        (($actionID == DELETE_MESSAGES || $actionID == MOVE_MESSAGES || $actionID == COPY_MESSAGES) &&
         ($HTTP_GET_VARS['array_index'] == count($sorted) - 2))) {
        if ($prefs->getValue('delhide')) {
            $sorted = imap_sort($imp['stream'], $prefs->getValue('sortby'),
                                $prefs->getValue('sortdir'), SE_UID, 'UNDELETED');
        } else {
            $sorted = imap_sort($imp['stream'], $prefs->getValue('sortby'),
                                $prefs->getValue('sortdir'), SE_UID);
        }
        $sorted = IMP_message::range($sorted, $HTTP_GET_VARS['array_index'] + $imp['offset'] + 1,
                                     $imp['offset'], $HTTP_GET_VARS['array_index'], $imp['msgcount']);
        $imp['msgl'] = implode(':', $sorted);
    }
}
$msgindex = $HTTP_GET_VARS['array_index'] + $imp['offset'];

/* Are we using printer-friendly formatting? */
$printer_friendly = false;

/* Run through action handlers */
switch ($actionID) {
 case NO_ACTION:
     break;
     
 case PRINT_MESSAGE:
     $printer_friendly = true;
     break;
     
 case UNDELETE_MESSAGES:
     if (!empty($HTTP_GET_VARS['index'])) {
         IMP_message::undelete(array($HTTP_GET_VARS['index']));
         $HTTP_GET_VARS['array_index']++;
         $msgindex++;
         $HTTP_GET_VARS['index'] = $sorted[$HTTP_GET_VARS['array_index']];
     }
     break;
     
 case DELETE_MESSAGES:
     if (!empty($HTTP_GET_VARS['index'])) {
         if (!IMP_message::delete(array($HTTP_GET_VARS['index']))) {
             $HTTP_GET_VARS['array_index']++;
             $msgindex++;
             $HTTP_GET_VARS['index'] = $sorted[$HTTP_GET_VARS['array_index']];
         } else {
             if ($conf['server']['use_trash']) {
                 /* Nuke the message we've just deleted from the
                    message array. */
                 $newsorted = array();
                 for ($i = 0; $i < count($sorted); $i++) {
                     if ($i != $HTTP_GET_VARS['array_index']) {
                         $newsorted[] = $sorted[$i];
                     }
                 }
                 $sorted = $newsorted;
                 $imp['msgl'] = implode(":",$sorted);
                 $HTTP_GET_VARS['index'] = $sorted[$HTTP_GET_VARS['array_index']];
             } else {
                 if ($imp['protocol'] == 'pop3') { 
                     /* expunge the mailbox to actually delete the
                        message, and then recalculate the message
                        list */
                     imap_expunge($imp['stream']);
                     $sorted = IMP_message::range(imap_sort($imp['stream'], $prefs->getValue('sortby'), $prefs->getValue('sortdir'), SE_UID), $HTTP_GET_VARS['array_index'] + $imp['offset'] + 1, $imp['offset'], $HTTP_GET_VARS['array_index'], $imp['msgcount']);
                     $imp['msgl'] = implode(':', $sorted);
                     $HTTP_GET_VARS['index'] = $sorted[$HTTP_GET_VARS['array_index']];
                     $msgindex = $HTTP_GET_VARS['array_index'] + $imp['offset'];
                 } elseif ($prefs->getValue('delhide')) {
                     $sorted = imap_sort($imp['stream'], $prefs->getValue('sortby'),
                                         $prefs->getValue('sortdir'), SE_UID, 'UNDELETED');
                     $sorted = IMP_message::range($sorted, $HTTP_GET_VARS['array_index'] + $imp['offset'] + 1,
                                                  $imp['offset'], $HTTP_GET_VARS['array_index'], $imp['msgcount']);
                     $imp['msgl'] = implode(':', $sorted);
                     $HTTP_GET_VARS['index'] = $sorted[$HTTP_GET_VARS['array_index']];
                 } else {
                     $HTTP_GET_VARS['array_index']++;
                     $msgindex++;
                     $HTTP_GET_VARS['index'] = $sorted[$HTTP_GET_VARS['array_index']];
                 }
             }
         }
     }
     break;

 case MOVE_MESSAGES:
     if (!empty($HTTP_GET_VARS['index']) && !empty($HTTP_GET_VARS['targetMbox'])) {
         IMP_message::copy($HTTP_GET_VARS['targetMbox'], array($HTTP_GET_VARS['index']), MOVE_MESSAGES);
         $HTTP_GET_VARS['array_index']++;
         $msgindex++;
         $HTTP_GET_VARS['index'] = $sorted[$HTTP_GET_VARS['array_index']];
     }
     break;

 case COPY_MESSAGES:
     if (!empty($HTTP_GET_VARS['index']) && !empty($HTTP_GET_VARS['targetMbox'])) {
         IMP_message::copy($HTTP_GET_VARS['targetMbox'], array($HTTP_GET_VARS['index']), COPY_MESSAGES);
         $HTTP_GET_VARS['array_index']++;
         $msgindex++;
         $HTTP_GET_VARS['index'] = $sorted[$HTTP_GET_VARS['array_index']];
     }
     break;
     
 case SPAM_REPORT:
     if (!$conf['spam']['reporting']) break;
     
     /* Fetch the raw message contents (headers and first 10k of body). */
     $raw_msg = imap_fetchheader($imp['stream'], $HTTP_GET_VARS['index'], FT_UID) . "\n" .
         substr(imap_body($imp['stream'], $HTTP_GET_VARS['index'], FT_UID), 0, 10240);
     
     /* Build the MIME structure. */
     $mime = new MimeMessage($raw_msg);
     $text_part = new MimePart();
     $text_part->type = 'text';
     $text_part->subtype = 'plain';
     $text_part->contents = $raw_msg;
     
     $mime->add_part($text_part);
     $msg = $mime->generate();
     
     $headers['Message-ID'] = '<' . uniqid(time() . '.') . '@' . $SERVER_NAME . '>';
     $headers['Date'] = date('r');
     $headers['To'] = $conf['spam']['email'];
     $headers['From'] = get_barefrom();
     $recipients = $conf['spam']['email'];
     $headers['Subject'] = _("Spam Report from") . ' ' . $imp['user'];
     $headers = $mime->header($headers);
     addSiteHeaders($headers);
     
     include_once 'Mail.php';
     $mailer = Mail::factory($conf['mailer']['type'], $conf['mailer']['params']);
     $mailer->send($recipients, $headers, $msg);
     break;
}

if ($conf['user']['allow_folders']) {
    $options = IMP::flistSelect(_("This message to"), true, array($imp['mailbox']));
}

/* If we grab the headers before grabbing body parts, we'll see when a message is unread */
$h = imap_header($imp['stream'], imap_msgno($imp['stream'], $HTTP_GET_VARS['index']));

/* parse MIME info */
if ($structure = imap_fetchstructure($imp['stream'], $HTTP_GET_VARS['index'], FT_UID)) {
    $bodypart = 1;
    $inline_parts = 0;
    $atc = '';
    $msg = '&nbsp;';
    $attachments = MIME_Structure::parse($structure, $HTTP_GET_VARS['index'], $MimeID);
    $msg = buildMessage($attachments, $msg);
} else {
    // FIXME: add a parameter here so that mailbox.php can explain the error
    header('Location: ' . Horde::url('mailbox.php', true));
    exit;
}

if ($conf['compress_pages']) {
    HTTP_Cache::start();
}

require $conf['paths']['templates'] . '/doctype.inc';

if ($conf['user']['allow_view_source']) {
    if (!isset($atc) || ($inline_parts == 1 && count($attachments) == 1)) {
        $atc = '';
    }
    $atc .= '<tr><td>';
    $atc .= Horde::img($mime_actions['message/source']['icon'], 'alt="message/source"');
    $atc .= '</td><td>&nbsp;</td><td>';
    $atc .= "<a onmouseover=\"status='" . _("View Raw Message Source") . "'; return true;\" onmouseout=\"status='';\" href=\"javascript:view('" . VIEW_SOURCE . "', 0);\">" . _("Message Source") . '</a>';
    $atc .= '</td><td>&nbsp;</td>';
    $atc .= '<td>' . _("Save as") . '</td><td><a onmouseover="status=\'' . _("Download message source") . '\'; return true;" onmouseout="status=\'\';" href="' . Horde::url('view.php?actionID=' . SAVE_MESSAGE . '&index=' . $HTTP_GET_VARS['index']) . '">';
    $atc .= Horde::img('download.gif', 'alt="' . _("Download") . '"');
    $atc .= "</a></td></tr>\n";
}

if (isset($h->date))     $dat = htmlspecialchars(chop($h->date));
if (isset($h->to))       $to  = buildAddressLinks($h->to);
if (isset($h->from))     $frm = buildAddressLinks($h->from);
if (isset($h->cc))       $cc  = buildAddressLinks($h->cc);
if (isset($h->bcc))      $bcc = buildAddressLinks($h->bcc);
if (isset($h->reply_to)) $rep = buildAddressLinks($h->reply_to);

if ((isset($h->subject)) && (strlen($h->subject) != 0)) {
    $sub = htmlspecialchars(MIME::decode($h->subject));

    /* filter the subject text, if requested */
    if ($prefs->getValue('filtering')) {
        $sub = Text::filter($sub, $conf['msg']['filtering']['words'],
            $conf['msg']['filtering']['replacement']);
    }
} else {
    $sub = _("[No subject]");
}

/* Set the status information of the message */
$status = '';
$is_deleted = false;
if ($imp['protocol'] != 'pop3') {
    if (!empty($h->toaddress) && strstr(strtolower($h->toaddress), strtolower(get_barefrom()))) $status .= $conf['mailbox']['personal_flag'];
    if ($h->Unseen == 'U' || $h->Recent == 'N') $status .= $conf['mailbox']['unseen_flag'];
    if ($h->Answered == 'A') $status .= $conf['mailbox']['answered_flag'];
    if (isset($h->Draft) && $h->Draft == 'X') $status .= $conf['mailbox']['draft_flag'];
    if ($h->Flagged == 'F') $status .= $conf['mailbox']['important_flag'];
    if ($h->Deleted == 'D') { 
        $status .= $conf['mailbox']['deleted_flag'];
        $is_deleted = true;
    }
}

$msg_index = $HTTP_GET_VARS['array_index'] + 1;

$title = $imp['label'] . ": $sub";
require $conf['paths']['templates'] . '/common-header.inc';

if (!$printer_friendly) {
	include './menu.php';
}

$prev_index = $HTTP_GET_VARS['array_index'] - 1;
$next_index = $HTTP_GET_VARS['array_index'] + 1;

if (isset($sorted[$prev_index])) {
    $prev = $sorted[$prev_index];
} else {
    $prev = -1;
}

if (isset($sorted[$next_index])) {
    $next = $sorted[$next_index];
} else {
    $next = -1;
}

if ($msg_index != count($sorted)) {
    $form_target = 'message.php';
    $form_ident  = 'index';
    $form_method = 'get';
} else {
    $form_target = 'mailbox.php';
    $form_ident  = 'indices';
    $form_method = 'post';
}

if (!$printer_friendly) {
    if ($browser->hasFeature('javascript')) {
        include './lib/js/open_print_win.js';
    }
    include $conf['paths']['templates'] . '/message/navbar_aux_top.inc';
    include $conf['paths']['templates'] . '/message/navbar.inc';
}
require $conf['paths']['templates'] . '/message/headers.inc';
require $conf['paths']['templates'] . '/message/message.inc';
if (!$printer_friendly) {
    include $conf['paths']['templates'] . '/message/navbar.inc';
    include $conf['paths']['templates'] . '/message/navbar_aux_bottom.inc';
}
if ($browser->hasFeature('javascript')) {
    require $conf['paths']['templates'] . '/message/javascript.inc';
}

if ($conf['user']['online_help'] && $browser->hasFeature('javascript')) {
    Help::javascript();
}
$registry->shutdown();

require $conf['paths']['templates'] . '/common-footer.inc';

$prefs->store();

if ($conf['compress_pages']) {
    HTTP_Cache::output();
}
?>
