<?php
/*
 * $Horde: imp/poppasswd.php,v 1.33 2000/11/07 22:21:42 chuck Exp $
 *
 * Copyright 1999, 2000 Oliver M. Bolzer <oliver@gol.com>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

require_once './lib/base.php';

/* read the current session; exit if there isn't a valid one */
session_name($horde['session_name']);
@session_start();

if (!isset($imp) || !is_array($imp)) {
    header('Location: ' . Horde::url('login.php?reason=logout', true));
    exit;
}
IMP::setupSession();

/* doctype */
require $conf['paths']['templates'] . '/doctype.inc';


/* Get the poppassd server from config; otherwise set to safe values */
if (isset($conf['passwd']['server'])) {
    $poppassd_server = $conf['passwd']['server'];
    $poppassd_port = $conf['passwd']['port'];
}
else {
    $poppassd_server = '/dev/null';
    $poppassd_port   = 0;
}

/* run through ActionIDs */
if (isset($actionID)) {

    switch ($actionID) {
    case NO_ACTION:
        break;
    
    case PASSWD_INPUT:
        include $conf['paths']['templates'] . '/passwd/poppassd_input.inc';
        break;
    
    case PASSWD_DISALLOW:
        $errortype = 'nopermission';
        include $conf['paths']['templates'] . '/passwd/poppassd_error.inc';
        break;
    
    case PASSWD_CHANGE:
        /* error checking */
        if (!($username == $imp['user'])) {
            $errortype = 'username';
            include $conf['paths']['templates'] . '/passwd/poppassd_error.inc';
        }
        if (strcmp($old_pass, Secret::read(IMP::getKey(), $imp['pass'])) != 0) {
            $errortype = 'old_pass';
            include $conf['paths']['templates'] . '/passwd/poppassd_error.inc';
        }
        if (strlen($new_pass) <= 4) {
            $errortype = 'short_pass';
            include $conf['paths']['templates'] . '/passwd/poppassd_error.inc';
        }
        if (strcmp($new_pass, $new_pass2) != 0) {
            $errortype = 'new_pass_ne';
            include $conf['paths']['templates'] . '/passwd/poppassd_error.inc';
        }
    
        /* OK, the input data seems to be OK, connect */
        $passd = fsockopen($poppassd_server, $poppassd_port, &$errno, &$errstr);
        if (!$passd) {
            $errortype = "Password Server: $errno($errstr)";
            include $conf['paths']['templates'] . '/passwd/poppassd_error.inc';
        }
        else {
            $input = fgets($passd, 128);
            if (!(substr($input, 0, 3) == 200)) {
                fclose($passd);
                $errortype = 'no_connect';
                include $conf['paths']['templates'] . '/passwd/poppassd_error.inc';
            }
            $output = 'user ' . $imp['user'] . "\n";
            fputs($passd, $output);
            
            $input = fgets($passd, 128);
            if (!(substr($input, 0, 3) == 200)) {
                fclose($passd);
                $errortype = $input;
                include $conf['paths']['templates'] . '/passwd/poppassd_error.inc';
            }
            $output = 'pass ' . $imp['pass'] . "\n";
            fputs($passd, $output);
            
            $input = fgets($passd, 128);
            if (!(substr($input, 0, 3) == 200)) {
                fclose($passd);
                $errortype = $input;
                include $conf['paths']['templates'] . '/passwd/poppassd_error.inc';
            }
            $output = "newpass $new_pass\n";
            fputs($passd, $output);
            
            $input = fgets($passd, 128);
            if (!(substr($input, 0, 3) == 200)) {
                fclose($passd);
                $errortype = $input;
                include $conf['paths']['templates'] . '/passwd/poppassd_error.inc';
            }
            $output = "quit\n";
            fputs($passd, $output);
            
            $input = fgets($passd, 128);
            if (!(substr($input, 0, 3) == 200)) {
                fclose($passd);
                $errortype = $input;
                include $conf['paths']['templates'] . '/passwd/poppassd_error.inc';
            }
            fclose($passd);
            
            /* yeah, password changed, user must re-login */
            echo '<script language=javascript>
                window.location="login.php?reason=chpass";
                </script>';
            exit;
        }
        break;
        
    default:
        include $conf['paths']['templates'] . '/passwd/poppassd_input.inc';
    } // switch
    
}
else {
    include $conf['paths']['templates'] . '/passwd/poppassd_input.inc';
}

?>
