<?php
/*
 * $Horde: imp/prefs.php,v 2.110 2001/01/10 18:33:23 jon Exp $
 *
 * Copyright 1999, 2000 Charles J. Hagenbuch <chuck@horde.org>
 * Copyright 1999, 2000 Jon Parise <jon@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

require_once './lib/base.php';
require_once '../lib/Registry.php';
require './config/servers.php';

$registry = new Registry();

/* Retrieve authentication information from session. */
session_name($horde['session_name']);
@session_start();

if (!isset($HTTP_SESSION_VARS['imp']) || !is_array($HTTP_SESSION_VARS['imp'])) {
    header('Location: ' . Horde::url('login.php?reason=logout', true));
    exit;
}
IMP::setupSession();
IMP::authenticate(OP_HALFOPEN);

$updated = false;

/* Run through the action handlers */
$actionID = IMP_util::getAction();
switch($actionID) {
    
 case NO_ACTION:
     break;
     
 case RETURN_TO_MAILBOX:
     header('Location: ' . Horde::url('mailbox.php', true));
     break;
     
 case UPDATE_PREFS:
     if (isset($HTTP_POST_VARS['fullname']) && $prefs->isChangeable('fullname')) {
         $prefs->setValue('fullname', Horde::dispelMagicQuotes($HTTP_POST_VARS['fullname']));
     }
     if (isset($HTTP_POST_VARS['signature']) && $prefs->isChangeable('signature')) {
         $prefs->setValue('signature', Horde::dispelMagicQuotes($HTTP_POST_VARS['signature']));
     }
     if ($prefs->isChangeable('sig_dashes')) {
         $prefs->setValue('sig_dashes', !empty($HTTP_POST_VARS['sig_dashes']) ? 1 : 0);
     }
     if ($prefs->isChangeable('sig_first')) {
         $prefs->setValue('sig_first', !empty($HTTP_POST_VARS['sig_first']) ? 1 : 0);
     }
     if (isset($HTTP_POST_VARS['new_lang']) && $prefs->isChangeable('language')) {
         $prefs->setValue('language', Horde::dispelMagicQuotes($HTTP_POST_VARS['new_lang']));
     }
     if (isset($HTTP_POST_VARS['folders']) && $prefs->isChangeable('folders')) {
         $prefs->setValue('folders', Horde::dispelMagicQuotes($HTTP_POST_VARS['folders']));
     }
     if (isset($HTTP_POST_VARS['from']) && $prefs->isChangeable('from_addr')) {
         $prefs->setValue('from_addr', Horde::dispelMagicQuotes($HTTP_POST_VARS['from']));
     }
     if (isset($HTTP_POST_VARS['replyto']) && $prefs->isChangeable('replyto_addr')) {
         $prefs->setValue('replyto_addr', Horde::dispelMagicQuotes($HTTP_POST_VARS['replyto']));
     }
     if (isset($HTTP_POST_VARS['sortby']) && $prefs->isChangeable('sortby')) {
         $prefs->setValue('sortby', $HTTP_POST_VARS['sortby']);
     }
     if (isset($HTTP_POST_VARS['sortdir']) && $prefs->isChangeable('sortdir')) {
         $prefs->setValue('sortdir', $HTTP_POST_VARS['sortdir']);
     }
     if (isset($HTTP_POST_VARS['max_msgs']) && $prefs->isChangeable('max_msgs')) {
         if (intval($HTTP_POST_VARS['max_msgs']) != $HTTP_POST_VARS['max_msgs']) {
             Horde::raiseMessage(_("The number of messages per page must be a number."), HORDE_ERROR);
         } elseif ($HTTP_POST_VARS['max_msgs'] == 0) {
             Horde::raiseMessage(_("The number of messages per page must be at least one."), HORDE_ERROR);
         } else {
             $prefs->setValue('max_msgs', $HTTP_POST_VARS['max_msgs']);
         }
     }
     if (isset($HTTP_POST_VARS['server']) && $conf['server']['server_list']) {
         if ($prefs->isChangeable('mailhost')) {
             $prefs->setValue('mailhost', $servers[$HTTP_POST_VARS['server']]['server']);
         }
         if ($prefs->isChangeable('mailport')) {
             $prefs->setValue('mailport', $servers[$HTTP_POST_VARS['server']]['port']);
         }
         if ($prefs->isChangeable('mailprotocol')) {
             $prefs->setValue('mailprotocol', $servers[$HTTP_POST_VARS['server']]['protocol']);
         }
     }
     if ($conf['user']['allow_folders']) {
         if ($prefs->isChangeable('subscribe')) {
	     if (((!empty($HTTP_POST_VARS['subscribe']) ? 1 : 0) != $prefs->getValue('subscribe'))
		 && isset($imp['imaptree'])) {
		 unset($imp['imaptree']);
	     }
             $prefs->setValue('subscribe', !empty($HTTP_POST_VARS['subscribe']) ? 1 : 0);
         }
            
         if (isset($HTTP_POST_VARS['sent_mail']) && $prefs->isChangeable('sent_mail_folder')) {
             if (empty($HTTP_POST_VARS['sent_mail']) && !empty($HTTP_POST_VARS['sent_mail_new'])) {
                 $HTTP_POST_VARS['sent_mail'] = $HTTP_POST_VARS['sent_mail_new'];
                 include_once './lib/folder.php';
                 IMP_folder::create($imp['stream'], preambleString() . $HTTP_POST_VARS['sent_mail'], $prefs->getValue('subscribe'));
             } elseif (substr($HTTP_POST_VARS['sent_mail'], 0, strlen(preambleString())) == preambleString()) {
                 $HTTP_POST_VARS['sent_mail'] = substr($HTTP_POST_VARS['sent_mail'], strlen(preambleString()));
             }
             $prefs->setValue('sent_mail_folder', Horde::dispelMagicQuotes($HTTP_POST_VARS['sent_mail']));
         }
            
         if ($prefs->isChangeable('save_sent_mail')) {
             $prefs->setValue('save_sent_mail', !empty($HTTP_POST_VARS['save_sent_mail']) ? 1 : 0);
         }
            
         if (isset($HTTP_POST_VARS['drafts']) && $prefs->isChangeable('drafts_folder')) {
             if (empty($HTTP_POST_VARS['drafts']) && !empty($HTTP_POST_VARS['drafts_new'])) {
                 $HTTP_POST_VARS['drafts'] = $HTTP_POST_VARS['drafts_new'];
                 include_once './lib/folder.php';
                 IMP_folder::create($imp['stream'], preambleString() . $HTTP_POST_VARS['drafts'], $prefs->getValue('subscribe'));
             } elseif (substr($HTTP_POST_VARS['drafts'], 0, strlen(preambleString())) == preambleString()) {
                 $HTTP_POST_VARS['drafts'] = substr($HTTP_POST_VARS['drafts'], strlen(preambleString()));
             }
             $prefs->setValue('drafts_folder', Horde::dispelMagicQuotes($HTTP_POST_VARS['drafts']));
         }
     }
     if (isset($HTTP_POST_VARS['num_words']) && $prefs->isChangeable('num_words')) {
         if (intval($HTTP_POST_VARS['num_words']) != $HTTP_POST_VARS['num_words']) {
             Horde::raiseMessage(_("The number of words per screen must be a number."), HORDE_ERROR);
         } elseif ($HTTP_POST_VARS['num_words'] == 0) {
             Horde::raiseMessage(_("The number of words per screen must be at least one."), HORDE_ERROR);
         } else {
             $prefs->setValue('num_words', $HTTP_POST_VARS['num_words']);
         }
     }
     if ($prefs->isChangeable('compose_popup')) {
         $prefs->setValue('compose_popup', !empty($HTTP_POST_VARS['compose_popup']) ? 1 : 0);
     }
     if ($prefs->isChangeable('filtering')) {
         $prefs->setValue('filtering', !empty($HTTP_POST_VARS['filtering']) ? 1 : 0);
     }
     if ($prefs->isChangeable('refresh_time')) {
         if (isset($HTTP_POST_VARS['refresh_time']) && ($HTTP_POST_VARS['refresh_time'] == 0 || in_array($HTTP_POST_VARS['refresh_time'], $conf['navigator']['times']))) { 
             $prefs->setValue('refresh_time', $HTTP_POST_VARS['refresh_time']);
         }
     }
     if ($prefs->isChangeable('nav_popup')) {
         $prefs->setValue('nav_popup', !empty($HTTP_POST_VARS['nav_popup']) ? 1 : 0);
     }
     if ($prefs->isChangeable('nav_expanded')) {
         $prefs->setValue('nav_expanded', !empty($HTTP_POST_VARS['nav_expanded']) ? 1 : 0);
     }
     if ($prefs->isChangeable('filter_on_login')) {
         $prefs->setValue('filter_on_login', !empty($HTTP_POST_VARS['filter_on_login']) ? 1 : 0);
     }
     
     $prefs->store();
     Horde::raiseMessage(_("Your preferences have been updated."), HORDE_MESSAGE);
     break;
}

/* Conditionally retrieve these values. */
if ($prefs->isChangeable('fullname')) {
    $fullname = $prefs->getValue('fullname');
}
if ($prefs->isChangeable('signature')) {
    $signature = $prefs->getValue('signature');
}
if ($prefs->isChangeable('sig_dashes')) {
    $sd_check = ($prefs->getValue('sig_dashes') == 0) ? '' : ' checked="checked"';
}
if ($prefs->isChangeable('sig_first')) {
    $sf_check = ($prefs->getValue('sig_first') == 0) ? '' : ' checked="checked"';
}
if ($prefs->isChangeable('folders')) {
    $folders = $prefs->getValue('folders');
}
if ($prefs->isChangeable('from_addr')) {
    $from = $prefs->getValue('from_addr');
}
if ($prefs->isChangeable('replyto_addr')) {
    $replyto = $prefs->getValue('replyto_addr');
}
if ($prefs->isChangeable('max_msgs')) {
    $max_msgs = $prefs->getValue('max_msgs');
}
if ($prefs->isChangeable('num_words')) {
    $numwords = $prefs->getValue('num_words');
}
if ($conf['user']['allow_folders']) {
    if ($prefs->isChangeable('sent_mail_folder')) {
        $sent_mail = $prefs->getValue('sent_mail_folder');
    }
    if ($prefs->isChangeable('drafts_folder')) {
        $drafts = $prefs->getValue('drafts_folder');
    }
    if ($prefs->isChangeable('subscribe')) {
        $subscribe = $prefs->getValue('subscribe');
        $subscribe_check = $subscribe ? ' checked="checked"' : '';
    }
}

/* 
 * Convert the refresh option times into human-readable form 
 */
$parsedTimes = array();
if ($prefs->isChangeable('refresh_time') && is_array($conf['navigator']['times'])) {
    foreach ($conf['navigator']['times'] as $i) {
        if ($i < 60) {
            $parsedTimes[$i] = "$i " . _('seconds');
        } else {
            $mn = round($i / 60);
            $parsedTimes[$i] = "$mn minute" . (($mn > 1) ? 's' : '');
        }
    }
}

/*
 * Build the options lists for the 'language' <select> dropdown list.
 */
if ($prefs->isChangeable('language')) {
    $language_options = '';
    foreach ($nls['languages'] as $key => $val) {
        $sel = ($key == $prefs->getValue('language')) ? ' selected="selected"' : '';
        $language_options .= "<option value=\"$key\"$sel>$val</option>\n";
    }
}

/*
 * Build the options list for the 'server' <select> dropdown list.
 */
if ($prefs->isChangeable('mailhost')) {
    $server_options = '';
    foreach ($servers as $key => $this_server) {
        $sel = ($this_server['server'] == $prefs->getValue('mailhost')) ? ' selected="selected"' : '';
        $server_options .= '<option value="' . $key . "\"$sel>";
        $server_options .= $this_server['name'] . "</option>\n";
    }
}

/*
 * Build the options list for the 'sortby' <select> dropdown list.
 */
if ($prefs->isChangeable('sortby')) {
    $sortby_fields = array(
        array(SORTARRIVAL, _("Arrival Time")),
        array(SORTDATE, _("Message Date")),
        array(SORTFROM, _("From Address")),
        array(SORTSUBJECT, _("Subject Field")),
        array(SORTSIZE, _("Message Size"))
    );
    $sortby_options = '';
    for ($i = 0; $i < count($sortby_fields); $i++) {
        $sel = ($prefs->getValue('sortby') == $sortby_fields[$i][0]) ? ' selected="selected"' : '';
        $sortby_options .= '<option value="' . $sortby_fields[$i][0] . "\"$sel>";
        $sortby_options .= $sortby_fields[$i][1];
        $sortby_options .= "</option>\n";
    }
}

/*
 * Build the options list for the 'sortdir' <select> dropdown list.
 */
if ($prefs->isChangeable('sortdir')) {
    $sortdir_fields = array(
        array(0, _("Ascending")),
        array(1, _("Descending"))
    );
    $sortdir_options = '';
    for ($i = 0; $i < count($sortdir_fields); $i++) {
        $sel = ($prefs->getValue('sortdir') == $sortdir_fields[$i][0]) ? ' selected="selected"' : '';
        $sortdir_options .= '<option value="' . $sortdir_fields[$i][0] . "\"$sel>";
        $sortdir_options .= $sortdir_fields[$i][1];
        $sortdir_options .= "</option>\n";
    }
}

/* Set the 'checked' toggle for the 'save sent mail' checkbox. */
if ($prefs->isChangeable('save_sent_mail')) {
    $ssm_check = ($prefs->getValue('save_sent_mail') == 0) ? '' : ' checked="checked"';
}

/* Set the 'checked' toggle for the 'compose popup' checkbox. */
if ($prefs->isChangeable('compose_popup')) {
    $cw_check = ($prefs->getValue('compose_popup') == 0) ? '' : ' checked="checked"';
}

/* Set the 'checked' toggle for the 'filtering' checkbox. */
if ($prefs->isChangeable('filtering')) {
    $filter_check = ($prefs->getValue('filtering') == 0) ? '' : ' checked="checked"';
}

/* Set the 'selected' toggle for the refresh_time selectbox. */
if ($prefs->isChangeable('refresh_time')) {
    $refresh_time_sel = $prefs->getValue('refresh_time');
}

/* Set the 'checked' toggle for the navigator section checkboxes. */
if ($prefs->isChangeable('nav_popup')) {
    $nav_popup_check = ($prefs->getValue('nav_popup') == 0) ? '' : ' checked="checked"';
}

if ($prefs->isChangeable('nav_expanded')) {
    $nav_xp_check = ($prefs->getValue('nav_expanded') == 0) ? '' : ' checked="checked"';
}

/* Set the 'checked' toggle for the filter on login checkbox. */
if ($prefs->isChangeable('filter_on_login')) {
    $fol_check = ($prefs->getValue('filter_on_login') == 0) ? '' : ' checked="checked"';
}

if ($conf['compress_pages']) {
    HTTP_Cache::start();
}

$title = _("User Preferences");
require $conf['paths']['templates'] . '/doctype.inc';
require $conf['paths']['templates'] . '/common-header.inc';
require './menu.php';
require $conf['paths']['templates'] . '/prefs/begin.inc';
require $conf['paths']['templates'] . '/prefs/info.inc';
include $conf['paths']['templates'] . '/prefs/buttons.inc';

if ($prefs->isChangeable('fullname') || $prefs->isChangeable('signature')) {
    include $conf['paths']['templates'] . '/prefs/personal.inc';
}

if ($prefs->isChangeable('language')) {
    include $conf['paths']['templates'] . '/prefs/language.inc';
}

if ($prefs->isChangeable('mailhost') || $prefs->isChangeable('folders')) {
    include $conf['paths']['templates'] . '/prefs/server.inc';
}

if ($prefs->isChangeable('from_addr') || $prefs->isChangeable('replyto_addr')) {
    include $conf['paths']['templates'] . '/prefs/addresses.inc';
}

if ($prefs->isChangeable('sortby') || $prefs->isChangeable('sortdir') ||
    $prefs->isChangeable('max_msgs')) {
    include $conf['paths']['templates'] . '/prefs/sorting.inc';
}

if (($imp['protocol'] != 'pop3') &&
    ($prefs->isChangeable('sent_mail_folder') ||
     $prefs->isChangeable('save_sent_mail') ||
     $prefs->isChangeable('drafts_folder'))) {
    include $conf['paths']['templates'] . '/prefs/folders.inc';
}

if ($prefs->isChangeable('compose_popup') || $prefs->isChangeable('num_words')) {
    include $conf['paths']['templates'] . '/prefs/compose.inc';
}

if ($prefs->isChangeable('filtering')) {
    include $conf['paths']['templates'] . '/prefs/viewing.inc';
}

if ($prefs->isChangeable('nav_popup') || $prefs->isChangeable('nav_expanded') ||$prefs->isChangeable('refresh_time')) {
    include $conf['paths']['templates'] . '/prefs/navigator.inc';
}

if ($conf['user']['allow_filters'] && ($prefs->isChangeable('filter_on_login') || $prefs->isChangeable('filter_on_login'))) {
    include $conf['paths']['templates'] . '/prefs/filters.inc';
}

include $conf['paths']['templates'] . '/prefs/buttons.inc';
include $conf['paths']['templates'] . '/prefs/end.inc';

$registry->shutdown();

include $conf['paths']['templates'] . '/common-footer.inc';

if ($conf['compress_pages']) {
    HTTP_Cache::output();
}
?>
