<?php
/*
 * $Horde: imp/spelling.php,v 2.23 2000/12/24 15:38:53 max Exp $
 *
 * Copyright 1999, 2000 Charles J. Hagenbuch <chuck@horde.org>
 * Copyright 1999, 2000 Jon Parise <jon@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

define('CHANGE',     1);
define('CHANGE_ALL', 2);
define('IGNORE',     3);
define('IGNORE_ALL', 4);

function find_offset($message, $word, $start)
{
    $offset = $start;
    $pos = -1;
    /* If all things are right the word is at offset - 1 */
    if ($start > 3) {
        $start -= 3;
    }
    else {
        $start = 0;
    }
    while ($pos == -1 && $start >= 0) {
        $pos = strpos($message,$word,$start);
        if ($pos == "" && !is_int($pos)) {
            $start--;
            $pos = -1;
        }
    }
    return $pos;
}                                

function estrtok(&$token, &$delim, $string, $separators, $init = 0)
{
    static $lastString = '';
    static $stringLen = 0;
    
    if ($init)
        $lastString = "";
    else {
        if ($lastString != $string) {
            $lastString = $string;
            $stringLen = strlen($string);
            $aStrTok = strtok($string, $separators);
            $stringLen -= strlen($aStrTok) + 1;
        } else {
            $aStrTok = strtok($separators);
            $stringLen -= strlen($aStrTok) + 1;
        }
    
        $token = $aStrTok;
        $pos = strlen($string) - $stringLen - 1;
        if ($pos < strlen($string))
            $delim = $string[$pos];
        else
            $delim = '';
    
        if (!$aStrTok)
            return $stringLen>=0;
        else
            return true;
    }
}

function replace_all($error, $replacement, $message)
{
    $msg = '';
    $tok = '';
    $delim = '';
    
    estrtok($tok,$delim,$message, "",1);
    while (estrtok($tok, $delim, $message, " <>@,.:;'\"&%!?()[]{}/|\n\\")) {
        if (trim($tok) == $error)
            $msg .= $replacement . $delim;
        else
            $msg .= $tok . $delim;
    }
    return $msg;
}

function highlight_error($error, $message, $offset)
{
    $msg = '';
    $tok = '';
    $delim = '';
    $pos = strpos($message,$error,($offset - 1 > 0 ? $offset - 1 : 0));
    if ($pos - 15 > 0) $start = $pos - 15;
    else $start = 0;
    
    $length = strlen($error) + 30;
    $message = substr($message,$start,$length);
    
    estrtok($tok,$delim,$message, "",1);
    while (estrtok($tok, $delim, $message, " <>@,.:;'\"&%!?()[]{}/|\n\\")) {
        if (trim($tok) == $error)
            $msg .= "<font color=\"ff0000\">$tok</font>$delim";
        else
            $msg .= $tok . $delim;
    }
    return $msg;
}

function member($string, $array)
{
    while (list($key, $val) = each($array)) {
        if (strtolower($val) == strtolower($string))
            return true;
    }
    return false;
}

if (isset($ispell) && isset($spell_lang)) {
    $spell_opt = $ispell[$spell_lang];
}
else {
    $spell_opt = '';
}

$numwords = $prefs->getValue('num_words');

if (isset($actionID) && $actionID == SPELL_CHECK_FORWARD) {
    if (isset($HTTP_POST_VARS['opt']) && !is_array($HTTP_POST_VARS['opt'])) 
        $HTTP_POST_VARS['opt'] = array($HTTP_POST_VARS['opt']);
    if (isset($HTTP_POST_VARS['subs']) && !is_array($HTTP_POST_VARS['subs'])) 
        $HTTP_POST_VARS['subs'] = array($HTTP_POST_VARS['subs']);
    if (isset($HTTP_POST_VARS['oldword']) && !is_array($HTTP_POST_VARS['oldword'])) 
        $HTTP_POST_VARS['oldword'] = array($HTTP_POST_VARS['oldword']);
    if (isset($HTTP_POST_VARS['subtext']) && !is_array($HTTP_POST_VARS['subtext'])) 
        $HTTP_POST_VARS['subtext'] = array($HTTP_POST_VARS['subtext']);
    if (isset($HTTP_POST_VARS['wordoffset']) && !is_array($HTTP_POST_VARS['wordoffset'])) 
        $HTTP_POST_VARS['subtext'] = array($HTTP_POST_VARS['wordoffset']);
    
    for ($i = 0; $i < count($HTTP_POST_VARS['opt']); $i++) {
        /* if they have an word with no suggestions and they
           dont type in a replacement, ignore it ... */
        $skipword = 0;
        if (isset($HTTP_POST_VARS['subtext'][$i]) && $HTTP_POST_VARS['subtext'][$i] != "")
            $replacement = $subtext[$i];
        else {
            if ($HTTP_POST_VARS['subs'][$i] == "0") {
                $ignoreall = $HTTP_POST_VARS['ignoreall'] . "," . $HTTP_POST_VARS['oldword'][$i];
                $skipword = 1;
            }
            else
                $replacement = $HTTP_POST_VARS['subs'][$i];
        }
        
        if (!$skipword) {
            $pos = -1;
            
            $realoffset = $HTTP_POST_VARS['wordoffset'][$i];
            /* Just in case things are whackily out ... */
            if ($realoffset > strlen($HTTP_POST_VARS['message'])) $realoffset = strlen($HTTP_POST_VARS['message']) - 1;
            
            $pos = find_offset($HTTP_POST_VARS['message'], $HTTP_POST_VARS['oldword'][$i], $realoffset);

            switch($HTTP_POST_VARS['opt'][$i]) {
            case IGNORE:
                $consume = $pos + strlen($HTTP_POST_VARS['oldword'][$i]);
                $addition = htmlspecialchars(substr($HTTP_POST_VARS['message'],0,$consume));
                $HTTP_POST_VARS['newmsg'] .= $addition;
                $HTTP_POST_VARS['newmsg'] = str_replace('"', '&quot;', $HTTP_POST_VARS['newmsg']);
                $HTTP_POST_VARS['message'] = substr($HTTP_POST_VARS['message'], $consume);
                /* Adjust offsets, as they could be wildly out */
                for ($msgnum = 0; $msgnum < count($HTTP_POST_VARS['wordoffset']); $msgnum++)
                    $HTTP_POST_VARS['wordoffset'][$msgnum] -= $consume;
                break;
            case IGNORE_ALL:
                $ignoreall = $HTTP_POST_VARS['ignoreall'] . "," . $HTTP_POST_VARS['oldword'][$i];
                break;
            case CHANGE_ALL:
                $HTTP_POST_VARS['message'] = replace_all($HTTP_POST_VARS['oldword'][$i], $replacement, $HTTP_POST_VARS['message']);
                break;
            case CHANGE:
                $ignores = explode(',', $HTTP_POST_VARS['ignoreall']);
                if (!member($HTTP_POST_VARS['oldword'][$i],$ignores)) {
                    /* Let's try and keep those offsets semi correct ...*/
                    $adjoffset = strlen($replacement) - strlen($HTTP_POST_VARS['oldword'][$i]);
                    for($msgnum = 0; $msgnum < count($HTTP_POST_VARS['wordoffset']); $msgnum++)
                        $HTTP_POST_VARS['wordoffset'][$msgnum] += $adjoffset;
                    
                    $tempmessage = substr($HTTP_POST_VARS['message'],0,$pos);
                    $tempmessage .= $replacement;
                    $tempmessage .= substr($HTTP_POST_VARS['message'],$pos + strlen($HTTP_POST_VARS['oldword'][$i]));
                    $HTTP_POST_VARS['message'] = $tempmessage;
                }
                break;
            }
        }
    }
} else {
    $HTTP_POST_VARS['oldmsg'] = $HTTP_POST_VARS['message'];
    
    /* Have to start another wordlist to incorporate into the spell check
       dictionary methinks ... */
    $ignoreall = 'com,electricmail,Cc,www,Jan,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec,Fwd,DNS,http,CC,ca,html,tm,COM,mmunity,Co,op,https,netscape,webmail,bcc,jpg,gif,email,Tel,ie,eg';
}

// special traitement depending on language (quotes are not equally treated by
// ispell in english and in french)
switch ($language) {
 case 'fr':
     // $tocheck = str_replace("'", "\\'", escapeShellCmd($message));
     // break;
 default:
     $tocheck = escapeShellCmd($HTTP_POST_VARS['message']);
}

exec("echo $tocheck | " . $conf['utils']['spellchecker'] . ' -a ' . $spell_opt, $warnings);

/* Some quotes get translated back for some unknown reason ... */
/* Change em back */
/* if(isset($newmsg))
   $newmsg = str_replace("\"","&quot",$newmsg);
*/

$msg = '';

for ($i = 0; $i < count($warnings); $i++) {
    if (substr($warnings[$i], 0, 1) == '&') {
        $parts = explode(': ', $warnings[$i]);
        $info = explode(' ', $parts[0]);
        if (preg_match('|^[A-Z]*$|', $info[1]))
            $ignoreall .= ',' . $info[1];
        else
            $error[] = array($info[1],$info[3],$parts[1]);
    }
    if (preg_match('|^#|', $warnings[$i])) {
        $info = explode(' ', $warnings[$i]);
        if (preg_match('|^[A-Z]*$|', $info[1]))
            $ignoreall .= ',' . $info[1];
        else
            $error[] = array($info[1], $info[2], '');
    }
}
?>
