<?php
/*
 * $Horde: imp/view.php,v 2.84 2001/01/10 19:24:22 chuck Exp $
 *
 * Copyright 1999, 2000 Charles J. Hagenbuch <chuck@horde.org>
 * Copyright 1999, 2000 Jon Parise <jon@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

require_once './lib/base.php';
require_once './lib/mimetypes.php';
require_once '../lib/MIME.php';
require './config/mime.php';

session_name($horde['session_name']);
if ($browser->hasQuirk('cache_ssl_downloads')
    && isset($HTTP_SERVER_VARS['HTTPS'])
    && ($HTTP_SERVER_VARS['HTTPS'] == 'on')) {
    session_cache_limiter('public');
}
@session_start();

if (!isset($HTTP_SESSION_VARS['imp']) || !is_array($HTTP_SESSION_VARS['imp'])) {
    header('Location: ' . Horde::url('login.php?reason=logout', true));
    exit;
}
IMP::setupSession();
IMP::authenticate();

$actionID = IMP_util::getAction();

if (isset($HTTP_GET_VARS['mime']) && ($actionID != SAVE_MESSAGE) && ($actionID != VIEW_SOURCE)) {
    $mime = SessionCache::getObject($HTTP_GET_VARS['mime']);
} else if (($actionID != SAVE_MESSAGE) && ($actionID != VIEW_SOURCE)) {
    include_once '../lib/MIME/Structure.php';
    $structure = imap_fetchstructure($imp['stream'], $HTTP_GET_VARS['index'], FT_UID);
    $attachments = MIME_Structure::parse($structure, $HTTP_GET_VARS['index'], $MimeID);
    $mime = $attachments[$HTTP_GET_VARS['id']];
}


/* Run through action handlers */
switch ($actionID) {
 case NO_ACTION:
     break;
     
 case DOWNLOAD_ATTACH:
     header('Content-Type: application/x-unknown-' . $mime_types[$mime->type] . '-' . $mime->subtype);
     
     /* This should force a save file dialog.  According to a note
      * in MSDN, the suggested filename should NOT be in quotes.
      *
      * For IE 5.5, we break the header in a special way that makes
      * things work. I don't really want to know. */
     if ($browser->hasQuirk('break_disposition_header')) {
         header('Content-Disposition: filename=' . MIME::decode($mime->name));
     } else {
         header('Content-Disposition: attachment; filename=' . MIME::decode($mime->name));
     }
     break;
     
 case VIEW_ATTACH:
     /* If this is a text/html part, then we need to scan for CIDs
      * embedded in it in order to display images that might be
      * inlined in other parts of the message. */
     if ($mime->type == TYPETEXT && $mime->subtype == 'html') {
         include_once '../lib/MIME/Structure.php';
         MIME_Structure::parse(imap_fetchstructure($imp['stream'], $HTTP_GET_VARS['index'], FT_UID), $HTTP_GET_VARS['index'], $MimeID);
     }
     
     if (isset($mime->conf['view_function'])) {
         header('Content-Type: text/html');
         header('Content-Disposition: inline; filename=' . MIME::decode($mime->name));
         $func = $mime->conf['view_function'];
         echo $func($mime);
         exit;
     }
	 
     header('Content-Type: ' . $mime_types[$mime->type] . '/' . $mime->subtype);
     header('Content-Disposition: inline; filename=' . MIME::decode($mime->name));
     break;
     
 case VIEW_SOURCE:
     $msg = imap_fetchheader($imp['stream'], $HTTP_GET_VARS['index'], FT_UID) . "\n" . imap_body($imp['stream'], $HTTP_GET_VARS['index'], FT_UID);
     header('Content-Type: text/plain');
     header('Content-Disposition: inline; filename=Message Source');
     header('Content-Length: ' . strlen($msg));
     echo $msg;
     exit; 

 case SAVE_MESSAGE:
     $h = imap_header($imp['stream'], imap_msgno($imp['stream'], $HTTP_GET_VARS['index']));
    
     if (!empty($h->subject)) {
         $name = preg_replace('|\W|', '_', MIME::decode($h->subject));
     } else {
         $name = 'message';
     }
     
     $from = '<>';
     if (isset($h->from[0])) {
         if (isset($h->from[0]->mailbox) && isset($h->from[0]->host)) {
             $from = $h->from[0]->mailbox . '@' . $h->from[0]->host;
         }
     }
     $date  = strftime('%a %b %d %H:%M:%S %Y', $h->udate);
     $body  = 'From ' . $from . ' ' . $date. "\n";
     $body .= imap_fetchheader($imp['stream'], $HTTP_GET_VARS['index'], FT_UID);
     $body .= imap_body($imp['stream'], $HTTP_GET_VARS['index'], FT_UID);
     $body  = str_replace("\r\n", "\n", $body);
     
     header('Content-Type: application/RFC822');
     
     /* This should force a save file dialog.  According to a note
      * in MSDN, the suggested filename should NOT be in quotes.
      *
      * For IE 5.5, we break the header in a special way that makes
      * things work. I don't really want to know. */
     if ($browser->hasQuirk('break_disposition_header')) {
         header("Content-Disposition: filename=$name");
     } else {
         header("Content-Disposition: attachment; filename=$name");
     }
     break;
}

if ($actionID != SAVE_MESSAGE) {
    $body = imap_fetchbody($imp['stream'], $HTTP_GET_VARS['index'], $HTTP_GET_VARS['id'], FT_UID);
    
    if ($mime->encoding == ENCBASE64) {
        $body = imap_base64($body);
    } elseif ($mime->encoding == ENCQUOTEDPRINTABLE && ($qprint_msg = imap_qprint($body))) {
        $body = $qprint_msg;
    }
}

header('Content-Length: ' . strlen($body));
echo $body;

?>
